/*  double_list_main_object - object and methods for double list - hpaned.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */

#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf.h>

#include "double_list_main_object.h"
#include "double_list_process_object.h"
#include "interface.h"
#include "popupmenu.h"
#include "query_editor.h"
#include "preferences.h"

static GObjectClass *parent_class = NULL;

/* main */
extern GConfClient *client;

/* refresh */
extern gchar tcp[12][BUFSIZ];

/* preferences */
extern preferences_window preferences;

/* interface */
extern main_window interface;

/* query_editor */
extern query_editor pf;

/* Static prototypes */
static void double_list_main_class_init (DoubleListMainClass *);
static void double_list_main_init (DoubleListMain *);
static void double_list_main_finalize (GObject *);
static guint double_list_main_get_number_lines_for_query (DoubleListMain *, guint);
static GObject *double_list_main_constructor (GType, guint, GObjectConstructParam *);
static void selection_changed_layout_table (GtkTreeSelection *, gpointer);
static void double_list_main_expand_query (DoubleListMain *, guint);
static void double_list_main_add_all_queries (DoubleListMain *);


/*******************************************************************************
 *                                                                             *
 *                               EXTERNAL FUNCTIONS                            *
 *                                                                             *
 *******************************************************************************/

GObject *double_list_main_new(void)
{
	return G_OBJECT (g_object_new(DOUBLE_LIST_MAIN_TYPE, NULL));
}

GType double_list_main_get_type(void)
{
    static GType type = 0;

    if (type == 0)
	{
		static const GTypeInfo info = {
	    sizeof(DoubleListMainClass),	/* class_size; */
	    NULL,		/* base_init */
	    NULL,		/* base_finalise */
	    (GClassInitFunc) double_list_main_class_init,	/* class_init */
	    NULL,		/* class_finalise */
	    NULL,		/* class_data */
	    sizeof(DoubleListMain),	/* instance_size; */
	    0,			/* n_preallocs */
	    (GInstanceInitFunc) double_list_main_init	/* instance_init */
		};
		
		type = g_type_register_static(G_TYPE_OBJECT, "DoubleListMain", &info, 0);
    }

    return type;
}


/*******************************************************************************
 *                                                                             *
 *                               VARIOUS FUNCTIONS                             *
 *                                                                             *
 *******************************************************************************/

/*
 * set status all columns or single column
 */
void double_list_main_set_status_columns(DoubleListMain *dlm, gint number_column)
{
    guint i, j, number_queries, number_tables;
    GtkTreeViewColumn *column;
    DoubleListProcess *dlp;
    layout2_elem *elem;

    number_queries = double_list_main_get_number_queries(dlm);
    for (i = 0; i < number_queries; i++)
	{
		dlp = g_ptr_array_index (dlm->array_queries, i);
		number_tables = double_list_process_get_number_of_tables (dlp);
		
		for (j = 0; j < number_tables; j++)
		{
			elem = double_list_process_get_table(dlp, j);
			
			column = gtk_tree_view_get_column(GTK_TREE_VIEW(elem->treeview), number_column);
			gtk_tree_view_column_set_visible(column, preferences.state_fields[number_column]);
		}
    }
}

/*
 * append row to query-list
 */
void 
double_list_main_add_row(DoubleListMain *dlm, guint index)
{
    guint i;
    gchar *elem;
    GtkTreeIter iter_command_list, child;
    layout2_elem *l2elem;
    gboolean make_new_element = TRUE;
	DoubleListProcess *query;
	
	query = g_ptr_array_index(dlm->array_queries, index);

    for (i = 0; i < query->list_command_name->len; i++)
	{
		if (g_ascii_strcasecmp(g_ptr_array_index(query->list_command_name, i), tcp[0]) == 0)
		{
			make_new_element = FALSE;
			l2elem = double_list_process_get_table(query,i);
			break;
		}
	}

    if (make_new_element == TRUE)
	{
		gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(dlm->treestore), &iter_command_list, NULL, index);

		/* new table for single command */
		l2elem = double_list_process_set_table(query);

		/* add command on left table */
		elem = g_strdup(tcp[0]);
		g_ptr_array_add(query->list_command_name, (gpointer) elem);

		gtk_tree_store_append(dlm->treestore, &child, &iter_command_list);
		gtk_tree_store_set(dlm->treestore, &child, 0, NULL, 1, elem, -1);
    }

    l2elem->lines++;
    gtk_list_store_append(GTK_LIST_STORE(l2elem->liststore), &l2elem->iter);
    gtk_list_store_set(l2elem->liststore, &l2elem->iter, 0, tcp[1], 1, tcp[2], 2, tcp[3], 3, tcp[4], 4, tcp[5], 5, tcp[6], 6, tcp[7], 7, tcp[8], 8, tcp[9], 9, tcp[10], 10, tcp[11], -1);
}


/*******************************************************************************
 *                                                                             *
 *                               QUERY FUNCTIONS                               *
 *                                                                             *
 *******************************************************************************/

/*
 * expand query
 */
static void
double_list_main_expand_query (DoubleListMain *dlm, guint index)
{
	GtkTreeIter iter;
	GtkTreePath *path;
	
	gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(dlm->treestore), &iter, NULL, index);
	
	/* expand row */
	path = gtk_tree_model_get_path (GTK_TREE_MODEL (dlm->treestore), &iter);
	gtk_tree_view_expand_row (interface.treeview_left_column, path, FALSE);
}


/*
 * check for new query
 */
gboolean 
double_list_main_add_query_check (DoubleListMain *dlm, const gchar *name)
{
	guint len,i;
	gchar *basename;
	
	basename = g_strdup (name);
	len = g_slist_length (dlm->queries);

	/* if name of query exists return FALSE */
	for (i=0; i<len; i++)
	{
		if (g_ascii_strcasecmp (g_slist_nth_data (dlm->queries, i),basename)==0)
		{
			g_free (basename);
			return FALSE;
		}
	}
	
	g_free (basename);
	return TRUE;
}

/*
 * add new query
 */
void 
double_list_main_add_query (DoubleListMain *dlm, const gchar *name)
{
	GtkTreeIter iter;
	GtkTreePath *path;
	gchar *basename, *boldbasename, *gconfpath;
	GdkPixbuf *icon_refresh;
	DoubleListProcess *query;
	
	basename = g_strdup(name);
	
	/* append name query-name list */
	dlm->queries = g_slist_append (dlm->queries, basename);
	
	/* save queries list */
	gconf_client_set_list(client, gconf_concat_dir_and_key(GCONF_PATH_GLOBAL, "list_queries"), GCONF_VALUE_STRING, dlm->queries, NULL);
	
	/* append treestore */
	boldbasename = g_strconcat ("<b>",basename,"</b>",NULL);
	gtk_tree_store_append(GTK_TREE_STORE(dlm->treestore), &iter, NULL);
	gtk_tree_store_set(dlm->treestore, &iter, 1, boldbasename, -1);
	g_free (boldbasename);
	
	/* make a new double_list_process object */
	query = double_list_process_new();
	g_ptr_array_add(dlm->array_queries, query);
	gtk_container_add (GTK_CONTAINER(dlm->vbox_paned), GTK_WIDGET(query));
	
	/***** set icon apply/cancel *****/

	gconfpath = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(basename, strlen(basename)));
	
	/* if "autorefresh-gconf-field" doesn't exist, make it! */
	if (gconf_client_get (client, gconf_concat_dir_and_key (gconfpath, "autorefresh"), NULL) != NULL )
		query->autorefresh = gconf_client_get_bool(client, gconf_concat_dir_and_key (gconfpath, "autorefresh"), NULL);
	else
	{
		query->autorefresh = TRUE;
		gconf_client_set_bool(client, gconf_concat_dir_and_key(gconfpath, "autorefresh"), TRUE, NULL);
	}
	
	if (query->autorefresh == TRUE)
		icon_refresh = gdk_pixbuf_new_from_file (PIXMAPS_PATH "apply.png", NULL);
	else
		icon_refresh = gdk_pixbuf_new_from_file (PIXMAPS_PATH "cancel.png", NULL);
	
	gtk_tree_store_set(dlm->treestore, &iter, 0, icon_refresh, -1);
	
	/* scroll at row of new query name */	
	path = gtk_tree_model_get_path (GTK_TREE_MODEL(dlm->treestore), &iter);
	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(interface.treeview_left_column), path, NULL, NULL, 0.0, 0.0);
	
	gtk_tree_path_free(path);
	g_object_unref (icon_refresh);
	g_free (gconfpath);
}

/*
 * add all queries:
 * this function is called just when double_list_main_object is created
 */
static void 
double_list_main_add_all_queries (DoubleListMain *dlm)
{
	GtkTreeIter iter;
	gchar *basename, *boldbasename, *gconfpath;
	GdkPixbuf *icon_refresh;
	DoubleListProcess *query;
	guint i, len;
	
	
	/* get queries names */
	dlm->queries = gconf_client_get_list (client, gconf_concat_dir_and_key (GCONF_PATH_GLOBAL, "list_queries"), GCONF_VALUE_STRING, NULL);

	len = g_slist_length (dlm->queries);

	for (i=0; i<len; i++)
	{
		basename = g_slist_nth_data (dlm->queries, i);
		gconfpath = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(basename, strlen(basename)));

		/* append treestore */
		boldbasename = g_strconcat ("<b>",basename,"</b>",NULL);
		gtk_tree_store_append(GTK_TREE_STORE(dlm->treestore), &iter, NULL);
		gtk_tree_store_set(dlm->treestore, &iter, 1, boldbasename, -1);
		g_free (boldbasename);
	
		/* make a new double_list_process object */
		query = double_list_process_new();
		g_ptr_array_add(dlm->array_queries, query);
		gtk_container_add (GTK_CONTAINER(dlm->vbox_paned), GTK_WIDGET(query));
			
		/***** set icon apply/cancel *****/
		query->autorefresh = gconf_client_get_bool(client, gconf_concat_dir_and_key (gconfpath, "autorefresh"), NULL);

		if (query->autorefresh == TRUE)
			icon_refresh = gdk_pixbuf_new_from_file (PIXMAPS_PATH "apply.png", NULL);
		else
			icon_refresh = gdk_pixbuf_new_from_file (PIXMAPS_PATH "cancel.png", NULL);
	
		gtk_tree_store_set(dlm->treestore, &iter, 0, icon_refresh, -1);
	
		g_object_unref (icon_refresh);
		g_free (gconfpath);
	}	
}

/*
 * remove query
 */
void 
double_list_main_remove_query (DoubleListMain *dlm, GtkTreePath *path, guint index)
{
	DoubleListProcess *query_obj;
	gchar *name_query;
	GtkTreeIter iter;
	
	/* remove name from list on left */
	gtk_tree_model_get_iter (GTK_TREE_MODEL(dlm->treestore), &iter, path);
	gtk_tree_store_remove(dlm->treestore, &iter);

	/* set name for query to remove */
	name_query = g_slist_nth_data (dlm->queries, index);
	if (pf.gconf_query_basename != NULL) g_free (pf.gconf_query_basename);
	pf.gconf_query_basename = g_strdup (name_query);
	
	/* remove name from list queries names */
	dlm->queries = g_slist_remove (dlm->queries, name_query);
	g_free (name_query);

	/* save queries list */
	gconf_client_set_list(client, gconf_concat_dir_and_key(GCONF_PATH_GLOBAL, "list_queries"), GCONF_VALUE_STRING, dlm->queries, NULL);
	
	/* destroy object */
	query_obj = g_ptr_array_index (dlm->array_queries, index);
	gtk_widget_destroy (GTK_WIDGET (query_obj));
	g_ptr_array_remove_index (dlm->array_queries, index);
	
	if (dlm->selected_query == index)
		dlm->selected_query = -1;
	
	gtk_tree_path_free (path);
}

/*
 * clear single query
 */
static void 
double_list_main_clean_query(DoubleListMain *dlm, guint index)
{
	GtkTreeIter iter_command_list, child;
	DoubleListProcess *dlp;
	gboolean tmp_autorefresh;
	guint index_query = index;
	
	/* destroy query-vbox */
	dlp = g_ptr_array_index (dlm->array_queries, index_query);
	tmp_autorefresh = dlp->autorefresh;
	gtk_widget_destroy (GTK_WIDGET(dlp));
	
	/* move iter to right query */
	gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(dlm->treestore), &iter_command_list, NULL, index_query);
	
	/* remove query-children */
	while (gtk_tree_model_iter_children (GTK_TREE_MODEL(dlm->treestore), &child, &iter_command_list))
	{
		gtk_tree_store_remove(dlm->treestore, &child);
	}
	
	/* make new query-vbox */
	dlp = double_list_process_new();
	dlp->autorefresh = tmp_autorefresh;
	dlm->array_queries->pdata[index_query] = dlp;
	gtk_container_add (GTK_CONTAINER(dlm->vbox_paned), GTK_WIDGET(dlp));
	gtk_box_reorder_child (GTK_BOX(dlm->vbox_paned), GTK_WIDGET(dlp), index_query);
}

/*
 * update single query
 */
void 
double_list_main_refresh_query(DoubleListMain *dlm, guint index)
{
	gchar *prompt;
	guint index_query = index;

	double_list_main_clean_query (dlm, index_query);

	if (pf.gconf_query_basename != NULL)
			g_free (pf.gconf_query_basename);

	pf.gconf_query_basename = g_strdup (g_slist_nth_data(dlm->queries,index_query));
	
	/* show message on status bar */
	prompt = g_strconcat (_("Refreshing query: "), pf.gconf_query_basename, NULL);
	gnome_appbar_pop(GNOME_APPBAR(interface.statusbar));
	gnome_appbar_push(GNOME_APPBAR(interface.statusbar), prompt);
	g_free (prompt);
	
	/* make the refresh */
	update_clist (index_query);

	double_list_main_expand_query (dlm, index_query);
	
	if (dlm->selected_query == index_query)
		dlm->selected_query = -1;
}

/*
 * refresh all queries
 */
void 
double_list_main_refresh_query_full(DoubleListMain *dlm)
{
	guint i, len;
	DoubleListProcess *dlp;
	gboolean state_search;
	GtkTextIter start, end;
	
	/* clear debug output text view */
	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER(interface.debug_output_text_buffer),&start, &end);
	gtk_text_buffer_delete (GTK_TEXT_BUFFER(interface.debug_output_text_buffer),&start, &end);
	
	/* start refreshing */
	len = g_slist_length (dlm->queries);

	for (i=0; i<len; i++)
	{
		dlp = g_ptr_array_index (dlm->array_queries, i);
		
		if (dlp->autorefresh == TRUE)
			double_list_main_refresh_query(dlm, i);
	}

	if (pf.gconf_query_basename != NULL)
	{
		g_free (pf.gconf_query_basename);
		pf.gconf_query_basename = NULL;
	}
	
	/* search sensitive */
	state_search = double_list_main_is_there_a_row (dlm);
	gtk_widget_set_sensitive (interface.hbox_search, state_search);
	
	/* show message on status bar */
	gnome_appbar_pop(GNOME_APPBAR(interface.statusbar));
	gnome_appbar_push(GNOME_APPBAR(interface.statusbar), _("Selected queries refreshed."));
}


/*******************************************************************************
 *                                                                             *
 *                             SELECTION FUNCTIONS                             *
 *                                                                             *
 *******************************************************************************/

/*
 * select process on query list (left treeview)
 */
void double_list_main_select_process_row_in_query (DoubleListMain *dlm, guint number_query, guint number_process)
{
	GtkTreeIter root, child;
	GtkTreeSelection *selection;
	GtkTreePath *path;
	
	double_list_main_expand_query (dlm, number_query);
	
	/* get root node */
	gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (dlm->treestore), &root, NULL, number_query);
	
	/* get child node */
	gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (dlm->treestore), &child, &root, number_process);
	
	/* select process row */
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(interface.treeview_left_column));
	gtk_tree_selection_select_iter (selection, &child);
	
	path = gtk_tree_model_get_path (GTK_TREE_MODEL(dlm->treestore), &child);
	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(interface.treeview_left_column), path, NULL, NULL, 0.0, 0.0);
	gtk_tree_path_free(path);
}


/*******************************************************************************
 *                                                                             *
 *                                GETTER FUNCTIONS                             *
 *                                                                             *
 *******************************************************************************/
 
 /*
 * is there at least a row?
 */
gboolean double_list_main_is_there_a_row (DoubleListMain * dlm)
{
	guint len, i;
	
	len = g_slist_length (dlm->queries);
	
	if (len == 0) return FALSE;
	
	for (i=0; i<len; i++)
	{
		if (double_list_main_get_number_processes_for_query (dlm, i) > 0)
			return TRUE;
	}
	
	return FALSE;
}
 
 /*
 * return number of lines for a query
 */
static guint double_list_main_get_number_lines_for_query (DoubleListMain * dlm, guint index_query)
{
	guint processes, total, i;
	
	processes = double_list_main_get_number_processes_for_query (dlm, index_query);
	
	total = 0;
	
	for (i=0; i<processes; i++)
	{
		total += double_list_main_get_number_lines_process_for_query (dlm, index_query, i);
	}

	return (total);
}

/*
 * return name of a process
 */
gchar *double_list_main_get_name_process_for_query (DoubleListMain * dlm, guint index_query, guint index_process)
{
	DoubleListProcess *dlp;
	
	dlp = g_ptr_array_index(dlm->array_queries, index_query);
	
	return (double_list_process_get_name_process(dlp, index_process));
}

/*
 * return number of lines for a process
 */
guint double_list_main_get_number_lines_process_for_query (DoubleListMain * dlm, guint index_query, guint index_process)
{
	DoubleListProcess *dlp;
	
	dlp = g_ptr_array_index(dlm->array_queries, index_query);

	return (double_list_process_get_number_lines_for_table(dlp, index_process));
}

/*
 * return number of queries
 */
guint double_list_main_get_number_queries (DoubleListMain * dlm)
{
	return (g_slist_length(dlm->queries));
}

/*
 * return name of a query
 */
gchar *double_list_main_get_name_query (DoubleListMain * dlm, guint index)
{
	return (g_slist_nth_data(dlm->queries,index));
}

/*
 * return number of processes for query
 */
guint double_list_main_get_number_processes_for_query (DoubleListMain * dlm, guint index)
{
	DoubleListProcess *dlp;
	
	dlp = g_ptr_array_index(dlm->array_queries, index);
	
	return (double_list_process_get_number_of_tables(dlp));
}

/*
 * return liststore for table
 */
GtkListStore *double_list_main_get_process_liststore_for_query (DoubleListMain * dlm, guint index_query, guint index_process)
{
	DoubleListProcess *dlp;
	
	dlp = g_ptr_array_index(dlm->array_queries, index_query);
	
	return (double_list_process_get_liststore_for_table(dlp, index_process));
}

/*
 * return treeview for table
 */
GtkTreeView *double_list_main_get_process_treeview_for_query (DoubleListMain * dlm, guint index_query, guint index_process)
{
	DoubleListProcess *dlp;
	
	dlp = g_ptr_array_index(dlm->array_queries, index_query);
	
	return (double_list_process_get_treeview_for_table(dlp, index_process));
}

/*******************************************************************************
 *                                                                             *
 *                               INTERNAL FUNCTIONS                            *
 *                                                                             *
 *******************************************************************************/

/*
 * signal: get row from menu
 */
static void
selection_changed_layout_table(GtkTreeSelection * selection, gpointer data)
{
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
	{
		static gchar buf[BUFSIZ];
		guint root;
		DoubleListMain *dlm;
		GtkTreePath *path;
		
		dlm = DOUBLE_LIST_MAIN (data);

		path = gtk_tree_model_get_path(model, &iter);
		root = gtk_tree_path_get_indices(path)[0];
		
		gnome_appbar_pop(GNOME_APPBAR(interface.statusbar));	/* ready to print on statusbar */
		
		/* no first trees, just sub nodes */
		if (gtk_tree_path_get_depth (path) > 1)
		{
			layout2_elem *elem;
			guint node;
			DoubleListProcess *dlp;

			node = gtk_tree_path_get_indices(path)[1];
			
			if (dlm->selected_query != -1)
			{
				/* hide 'query' and table */
				dlp = g_ptr_array_index(dlm->array_queries, dlm->selected_query);
				gtk_widget_hide(GTK_WIDGET(dlp));
				
				if (double_list_process_get_number_of_tables (dlp) > 0)
				{
					elem = double_list_process_get_table(dlp, dlm->selected_elem);
					gtk_widget_hide(elem->scrolledwindow);
				}
			}
			
			dlm->selected_query = root;	/* save root */
			dlm->selected_elem = node;	/* save node */

			/* show 'query' and table */
			dlp = g_ptr_array_index(dlm->array_queries,root);
			gtk_widget_show(GTK_WIDGET(dlp));
			elem = double_list_process_get_table(dlp,node);
			gtk_widget_show(elem->scrolledwindow);

			/* show line numbers on status bar */
			if (elem->lines == 1)
				g_snprintf(buf, BUFSIZ, _("%d line for this process."), elem->lines);
			else
				g_snprintf(buf, BUFSIZ, _("%d lines for this process."), elem->lines);
    		}
    		else
    		{
    			guint total, processes;
    			
    			processes = double_list_main_get_number_processes_for_query (dlm, root);
    			total = double_list_main_get_number_lines_for_query (dlm, root);
    			
    			if ((processes != 1) && (total != 1))
    				g_snprintf(buf, BUFSIZ, _("This query has %d processes and a total of %d lines."), processes, total);
    			else if ((processes != 1) && (total == 1))
    				g_snprintf(buf, BUFSIZ, _("This query has %d processes and a total of %d line."), processes, total);
    			else if ((processes == 1) && (total != 1))
    				g_snprintf(buf, BUFSIZ, _("This query has %d process and a total of %d lines."), processes, total);
    			else
    				g_snprintf(buf, BUFSIZ, _("This query has %d process and a total of %d line."), processes, total);
    		}
		
		gnome_appbar_push(GNOME_APPBAR(interface.statusbar), &buf[0]);	/* flush on statusbar */
		
		gtk_tree_path_free(path);
		
		
	}
}				/* _selection_changed */


/*
 * callback for double-click on row
 */
void
view_onRowActivated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *col, gpointer userdata)
{
	GtkTreeSelection *selection;
	GtkTreeIter iter;
	DoubleListProcess *dlp;
	DoubleListMain *dlm = userdata;
	
	/* path from selected row */
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	gtk_tree_selection_select_path (selection, path);
	
	if (gtk_tree_path_get_depth (path) == 1)
	{
		guint index;
		GdkPixbuf *icon_refresh;
		gchar *gconfpath;
		
		index = gtk_tree_path_get_indices(path)[0];
		dlp = g_ptr_array_index(dlm->array_queries, index);
		dlp->autorefresh = !(dlp->autorefresh);
		
		/* save flag on gconf */
		gconfpath = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(g_slist_nth_data(dlm->queries,index), strlen(g_slist_nth_data(dlm->queries,index))));
		gconf_client_set_bool(client, gconf_concat_dir_and_key(gconfpath, "autorefresh"), dlp->autorefresh, NULL);
		g_free (gconfpath);
		
		/* image loader */
		if (dlp->autorefresh == TRUE)
			icon_refresh = gdk_pixbuf_new_from_file (PIXMAPS_PATH "apply.png", NULL);
		else
			icon_refresh = gdk_pixbuf_new_from_file (PIXMAPS_PATH "cancel.png", NULL);
		
		gtk_tree_model_get_iter (GTK_TREE_MODEL(dlm->treestore), &iter, path);
		gtk_tree_store_set(dlm->treestore, &iter, 0, icon_refresh, -1);
	}
}


/*******************************************************************************
 *                                                                             *
 *                  INTERNAL FUNCTIONS: CLASS-OBJECT DEFINITIONS               *
 *                                                                             *
 *******************************************************************************/


static void double_list_main_class_init(DoubleListMainClass * gclass)
{
    GObjectClass *object_class = G_OBJECT_CLASS(gclass);

    parent_class = g_type_class_peek_parent(gclass);
    object_class->finalize = double_list_main_finalize;
}

static void double_list_main_finalize(GObject * obj)
{
    DoubleListMain *dlm = (DoubleListMain *) obj;
    gint i;

    g_return_if_fail(obj != NULL);
    g_return_if_fail(DOUBLE_LIST_MAIN(obj));
    
    FREE_LIST (dlm->queries);
    
    g_ptr_array_foreach (dlm->array_queries, (GFunc)gtk_widget_destroy, NULL);
    g_ptr_array_free (dlm->array_queries, TRUE);

    /* Chain up to the parent class */
    G_OBJECT_CLASS(parent_class)->finalize(obj);
}

static void double_list_main_init(DoubleListMain * dlm)
{
    GtkCellRenderer *cell_left_command_column;
    GtkTreeViewColumn *column;
    GtkTreeSelection *treeselection;
    GtkTreeIter iter;
    
    /* make objects array */
    dlm->array_queries = g_ptr_array_new();
    dlm->selected_query = -1;
    
    /* tree view */
    treeselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(interface.treeview_left_column));
    gtk_tree_selection_set_mode(treeselection, GTK_SELECTION_SINGLE);
    dlm->treestore = gtk_tree_store_new(2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
    gtk_tree_view_set_model(GTK_TREE_VIEW(interface.treeview_left_column), GTK_TREE_MODEL(dlm->treestore));
    
    /* this column for expander */
    column = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title (column, _("Query"));
    
    cell_left_command_column = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_start(column, cell_left_command_column, FALSE);
    gtk_tree_view_column_set_attributes(column, cell_left_command_column, "pixbuf", 0, NULL);
    
    cell_left_command_column = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(column, cell_left_command_column, TRUE);
    gtk_tree_view_column_set_attributes(column, cell_left_command_column, "markup", 1, NULL);
    
    gtk_tree_view_append_column(GTK_TREE_VIEW(interface.treeview_left_column), column);
    
    /* right main vbox */
    dlm->vbox_paned = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (dlm->vbox_paned);
    gtk_paned_add2 (GTK_PANED(interface.hpaned), dlm->vbox_paned);
    
    double_list_main_add_all_queries (dlm);

    /* signals */
    g_signal_connect(G_OBJECT(treeselection), "changed", G_CALLBACK(selection_changed_layout_table), (gpointer) dlm);
    g_signal_connect(G_OBJECT(interface.treeview_left_column), "button-press-event", G_CALLBACK(view_onButtonPressed), NULL);
    g_signal_connect(G_OBJECT(interface.treeview_left_column), "row-activated", G_CALLBACK(view_onRowActivated), (gpointer) dlm);
}
