/*  double_list_process_object - object and methods for double list - single process.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */

#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "double_list_process_object.h"
#include "popupmenu.h"
#include "preferences.h"

static GObjectClass *parent_class = NULL;

/* Static prototypes */
static void double_list_process_class_init(gpointer, gpointer);
static void double_list_process_init(GTypeInstance *, gpointer);
static layout2_elem *new_layout2_table(void);
static void double_list_process_finalize(GtkObject *);

/* interface */
extern gchar *name_fields[];

/* view_options */
extern preferences_window preferences;


/*******************************************************************************
 *                                                                             *
 *                               EXTERNAL FUNCTIONS                            *
 *                                                                             *
 *******************************************************************************/

GtkWidget *double_list_process_new(void)
{
    return GTK_WIDGET(g_object_new(double_list_process_get_type(), NULL));
}

GType double_list_process_get_type(void)
{
    static GType type = 0;

    if (!type) {
	static const GTypeInfo info = {
	    sizeof(DoubleListProcessClass),	/* class_size; */
	    NULL,		/* base_init */
	    NULL,		/* base_finalise */
	    double_list_process_class_init,	/* class_init */
	    NULL,		/* class_finalise */
	    NULL,		/* class_data */
	    sizeof(DoubleListProcess),	/* instance_size; */
	    0,			/* n_preallocs */
	    double_list_process_init	/* instance_init */
	};

	type = g_type_register_static(GTK_TYPE_VBOX, "DoubleListProcess", &info, 0);
    }

    return type;
}

/*
 * return new element for list_layout2_table
 */
layout2_elem *double_list_process_set_table(DoubleListProcess * dlp)
{
    layout2_elem *l2elem = new_layout2_table();	/* make all widgets for table */
    g_ptr_array_add(dlp->list_layout2_table, l2elem);
    gtk_container_add(GTK_CONTAINER(dlp), l2elem->scrolledwindow);

    return l2elem;
}


/*******************************************************************************
 *                                                                             *
 *                                GETTER FUNCTIONS                             *
 *                                                                             *
 *******************************************************************************/

/*
 * return name process
 */
gchar *double_list_process_get_name_process(DoubleListProcess * dlp, guint i)
{
    return g_ptr_array_index(dlp->list_command_name, i);
}

/*
 * return element from list_layout2_table
 */
layout2_elem *double_list_process_get_table(DoubleListProcess * dlp, guint i)
{
    return g_ptr_array_index(dlp->list_layout2_table, i);
}

/*
 * return number of tables list_layout2_table
 */
guint double_list_process_get_number_of_tables(DoubleListProcess *dlp)
{
    return dlp->list_layout2_table->len;
}

/*
 * return number of tables list_layout2_table
 */
guint double_list_process_get_number_lines_for_table(DoubleListProcess *dlp, guint index)
{
	layout2_elem *elem;
	
	elem = double_list_process_get_table(dlp, index);
    return (elem->lines);
}

/*
 * return liststore of a table
 */
GtkListStore *double_list_process_get_liststore_for_table(DoubleListProcess *dlp, guint index)
{
	layout2_elem *elem;
	
	elem = double_list_process_get_table(dlp, index);
    return (elem->liststore);
}

/*
 * return treeview of a table
 */
GtkTreeView *double_list_process_get_treeview_for_table(DoubleListProcess *dlp, guint index)
{
	layout2_elem *elem;
	
	elem = double_list_process_get_table(dlp, index);
    return (GTK_TREE_VIEW (elem->treeview));
}

/*******************************************************************************
 *                                                                             *
 *                              INTERNAL FUNCTIONS                             *
 *                                                                             *
 *******************************************************************************/

static void double_list_process_finalize(GtkObject * obj)
{
    DoubleListProcess *dlp = (DoubleListProcess *) obj;
    gint i;

    g_return_if_fail(obj != NULL);
    g_return_if_fail(DOUBLE_LIST_PROCESS(obj));

    if (dlp->list_layout2_table == NULL) return;

    /* free list_layout2_table array */
    i = dlp->list_layout2_table->len - 1;

    if (i >= 0)
	{
		for (; i >= 0; i--) g_free(g_ptr_array_index(dlp->list_layout2_table, i));

		g_ptr_array_free(dlp->list_layout2_table, TRUE);
		dlp->list_layout2_table = NULL;
    }

    /* Chain up to the parent class */
    GTK_OBJECT_CLASS(parent_class)->destroy(obj);
}

static void double_list_process_class_init(gpointer gclass, gpointer data)
{
    GtkObjectClass *gtk_object_class = (GtkObjectClass *) gclass;

    parent_class = g_type_class_peek_parent(gclass);

    gtk_object_class->destroy = double_list_process_finalize;
}

static void double_list_process_init(GTypeInstance * object,
				     gpointer gclass)
{
    DoubleListProcess *dlp = DOUBLE_LIST_PROCESS(object);

	dlp->list_command_name = g_ptr_array_new();
    dlp->list_layout2_table = g_ptr_array_new();
}

static layout2_elem *new_layout2_table(void)
{
    GtkTreeSelection *selection;
    GtkCellRenderer *cell;
    GtkTreeViewColumn *column;
    layout2_elem *tmp_elem;

    int i = 0;

    tmp_elem = g_malloc(sizeof(layout2_elem));

    /* set '0' number of lines for single object */
    tmp_elem->lines = 0;

    /* make main column-list */
    tmp_elem->treeview = gtk_tree_view_new();
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tmp_elem->treeview));

    gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
    tmp_elem->liststore = gtk_list_store_new(11,
					     G_TYPE_STRING, G_TYPE_STRING,
					     G_TYPE_STRING, G_TYPE_STRING,
					     G_TYPE_STRING, G_TYPE_STRING,
					     G_TYPE_STRING, G_TYPE_STRING,
					     G_TYPE_STRING, G_TYPE_STRING,
					     G_TYPE_STRING);

    /* white - grey alternate */
    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tmp_elem->treeview), TRUE);
    gtk_tree_view_set_model(GTK_TREE_VIEW(tmp_elem->treeview), GTK_TREE_MODEL(tmp_elem->liststore));

    for (i = 0; i < 11; i++)
	{
		cell = gtk_cell_renderer_text_new();
		column = gtk_tree_view_column_new_with_attributes(name_fields[i + 1], cell, "text", i, NULL);
		gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
		gtk_tree_view_append_column(GTK_TREE_VIEW(tmp_elem->treeview), column);
    }
	
	for (i = 0; i < 10; i++)
	{
		column = gtk_tree_view_get_column(GTK_TREE_VIEW(tmp_elem->treeview), i);
	    gtk_tree_view_column_set_visible(column, preferences.state_fields[i]);
	}
		

    gtk_widget_show(tmp_elem->treeview);

    /* scrolled window */
    tmp_elem->scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(tmp_elem->scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(tmp_elem->scrolledwindow), GTK_SHADOW_OUT);
    gtk_container_add(GTK_CONTAINER(tmp_elem->scrolledwindow), tmp_elem->treeview);
	
    return tmp_elem;
}
