/*  graphics_stats - Window for graphical stats.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <glade/glade.h>

#include "graphics_stats.h"
#include "interface.h"
#include "double_list_main_object.h"
#include "bar_chart.h"

#define GCONF_PREFERENCES_PATH	"/apps/glsof/preferences/"

/* FUNCTIONS PROTOTYPES */
static void fill_stats_list (guint);

/* interface */
extern main_window interface;

/* extern */
graphics_stats_window stats;


/****************************************
 *          INTERNAL FUNCTIONS          *
 ****************************************/


static void
perc_cell_data_func    (GtkTreeViewColumn *col,
                        GtkCellRenderer   *renderer,
                        GtkTreeModel      *model,
                        GtkTreeIter       *iter,
                        gpointer           user_data)
   {
     gdouble perc;
     gchar   buf[20];

     gtk_tree_model_get(model, iter, 2, &perc, -1);

     g_snprintf(buf, sizeof(buf), "%.3g", perc);

     g_object_set(renderer, "text", buf, NULL);
   }
   

/****************************************
 *          EXTERNAL FUNCTIONS          *
 ****************************************/


static void fill_stats_list (guint index)
{
	GtkTreeIter iter;
	guint len, lines, total_lines, index_query, i;
	gdouble percent;
	
	index_query = index;
	total_lines = 0;
	
	len = double_list_main_get_number_processes_for_query (DOUBLE_LIST_MAIN(interface.dlm), index_query);
	
	for (i=0; i < len; i++)
		total_lines += double_list_main_get_number_lines_process_for_query (DOUBLE_LIST_MAIN(interface.dlm), index_query, i);

	for (i=0; i < len; i++)
	{
		lines = double_list_main_get_number_lines_process_for_query (DOUBLE_LIST_MAIN(interface.dlm), index_query, i);
		percent = (gdouble) (lines) / (gdouble) (total_lines);
		
		gtk_list_store_append(GTK_LIST_STORE(stats.liststore), &iter);
		gtk_list_store_set(GTK_LIST_STORE(stats.liststore), &iter, 
		0, double_list_main_get_name_process_for_query (DOUBLE_LIST_MAIN(interface.dlm), index_query, i), 
		1, percent,	/* bar */ 
		2, percent * 100,
		3, lines,
		-1);
	}
}

void on_stats_activate (GtkButton * button, gpointer user_data)
{
    GladeXML *xml_glade;
    GtkTreeStore *liststore;
	GtkTreeSelection *treeselection;
	GtkCellRenderer *cell;
    GtkTreeViewColumn *column;
	GtkTreeSortable *sortable;
	guint len;
	
    xml_glade =	glade_xml_new(GLADE_PATH "glsof_stats.glade", NULL, NULL);

    stats.window = glade_xml_get_widget(xml_glade, "glsof-stats");
    stats.scrolledwindow = glade_xml_get_widget(xml_glade, "scrolledwindow1");
	
	/* COLUMN LIST */
	stats.treeview = gtk_tree_view_new();
	treeselection =	gtk_tree_view_get_selection(GTK_TREE_VIEW(stats.treeview));

    gtk_tree_selection_set_mode(treeselection, GTK_SELECTION_SINGLE);
	
	stats.liststore = gtk_list_store_new(4,	G_TYPE_STRING, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_UINT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(stats.treeview), GTK_TREE_MODEL(stats.liststore));
	
	/* sortable list */
	sortable = GTK_TREE_SORTABLE(stats.liststore);

    //g_object_unref(G_OBJECT(stats.liststore));
	
	/* first column */
	cell = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(_("Process"), cell, "text", 0, NULL);
	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
	gtk_tree_view_column_set_sort_column_id(column, 0);
	gtk_tree_view_append_column(GTK_TREE_VIEW(stats.treeview), column);
	
	/* second column */
	cell = bar_chart_new();
	column = gtk_tree_view_column_new_with_attributes(_("Bars"), cell, "value", 1, NULL);
	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
	gtk_tree_view_column_set_sort_column_id(column, 1);
	gtk_tree_view_append_column(GTK_TREE_VIEW(stats.treeview), column);
	/* auto sorting column */
	gtk_tree_sortable_set_sort_column_id(sortable, 1, GTK_SORT_DESCENDING);
			
	/* third column */
	cell = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(_(" % "), cell, "text", 2, NULL);
	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
	gtk_tree_view_column_set_sort_column_id(column, 2);
	gtk_tree_view_append_column(GTK_TREE_VIEW(stats.treeview), column);
	/* automatic write %-form */
	gtk_tree_view_column_set_cell_data_func(column, cell, perc_cell_data_func, NULL, NULL);
			
	/* fourth column */
	cell = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(_("Number of files"), cell, "text", 3, NULL);
	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
	gtk_tree_view_column_set_sort_column_id(column, 3);
	gtk_tree_view_append_column(GTK_TREE_VIEW(stats.treeview), column);
	
	/* show list */
	gtk_widget_show(stats.treeview);
    gtk_container_add(GTK_CONTAINER(stats.scrolledwindow), stats.treeview);
	
	/* fill list */
	fill_stats_list (GPOINTER_TO_INT(user_data));
	
	/* exit glade */
    glade_xml_signal_autoconnect(xml_glade);
    g_object_unref(G_OBJECT(xml_glade));
}
