/*  interface - make main window and relative controls.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>
#include <libgnome/gnome-help.h>

#include "file_selection.h"
#include "interface.h"
#include "preferences.h"
#include "autorefresh.h"
#include "about.h"
#include "refresh.h"
#include "popupmenu.h"
#include "double_list_main_object.h"

const gchar *name_fields[] = {
    "COMMAND       ", "PID    ", "PGID    ", "PPID  ", "USER      ",    "FD   ", \
    "TYPE      ", "DEVICE    ", "SIZE     ", "NLINK   ", "NODE  ", "NAME"
};


/* main */
extern GConfClient *client;
extern gchar *lsof_path;

/* search */
extern gchar *name;
extern gchar *name_copy;

/* autorefresh */
extern gchar *filename;

/* extern */
main_window interface;
GtkWidget *splash_window = NULL;

/*******************************************************************************
 *                                                                             *
 *                           SIGNALS FOR MANUAL SECTIONS                       *
 *                                                                             *
 *******************************************************************************/
 
/*
 * call from contents-page
 */

void on_contents_general_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	GError *error = NULL;
	
	gnome_help_display ("glsof.xml", NULL, &error);
	
	if (error != NULL)
        {
                g_warning (error->message);

                g_error_free (error);
        }
}


/*******************************************************************************
 *                                                                             *
 *                           SHOW & HIDE FRAMES AND TABS                       *
 *                                                                             *
 *******************************************************************************/

/* 
 * autorefresh frame show & hide
 */
void on_autorefresh_button_clicked(GtkButton * button, gpointer user_data)
{
    if (GTK_WIDGET_VISIBLE(interface.autorefresh_frame))
	{
		gtk_widget_hide(interface.autorefresh_frame);
		gtk_button_set_label(GTK_BUTTON(interface.autorefresh_button), _("Show Autorefresh >>"));
    }
	else
	{
		gtk_widget_show(interface.autorefresh_frame);
		gtk_button_set_label(GTK_BUTTON(interface.autorefresh_button),  _("<< Hide Autorefresh"));
    }
}

/* 
 * show & hide "debug output text view" and "queries"
 */
void on_debug_output_button_clicked(GtkButton * button, gpointer user_data)
{
    if (gtk_notebook_get_current_page (GTK_NOTEBOOK(interface.main_window_notebook)) == 0)
	{
		gtk_notebook_set_current_page (GTK_NOTEBOOK(interface.main_window_notebook), 1);
		gtk_button_set_label(GTK_BUTTON(interface.debug_button), _("Queries output"));
    }
	else
	{
		gtk_notebook_set_current_page (GTK_NOTEBOOK(interface.main_window_notebook), 0);
		gtk_button_set_label(GTK_BUTTON(interface.debug_button),  _("Debug output"));
    }
}


/*
 * timeout for bounce-bar
 */
static gboolean progress_timeout(gpointer data)
{
    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(interface.statusbar_pb));

    return TRUE;
}

/*
 * refresh button clicked
 */
void on_refresh_clicked(GtkButton *button, gpointer user_data)
{
    guint bbtimeout;
    
    if (lsof_path == NULL)
    {
    	GtkWidget *dialog;

	dialog = gtk_message_dialog_new(GTK_WINDOW(interface.window), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
					_("Lsof was not found.\nPlease, set lsof-path in 'Edit->Preferences'."));

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	
	return;
    }
    
    /* No sensitive when refresh query */
    gtk_widget_set_sensitive(interface.vbox, FALSE);

    /* Add a timer callback to update the value of the progress bar */
    bbtimeout = g_timeout_add(100, progress_timeout, NULL);
    
    double_list_main_refresh_query_full(DOUBLE_LIST_MAIN(interface.dlm));	/* refresh all queries */

    /* remove bounce-bar timer */
    g_source_remove(bbtimeout);
    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(interface.statusbar_pb), 0.0);
    
    /* Main Window Sensitive */
    gtk_widget_grab_focus(interface.treeview_left_column);
    gtk_widget_set_sensitive(interface.vbox, TRUE);
}


/* 
 * destroy main interface and free mem allocated 
 */
void on_main_window_destroy(GtkObject * object, gpointer user_data)
{
    gint i;

    /* free strings */
    if (name_copy != NULL) g_free(name_copy);
    if (name != NULL) g_free(name);
    if (filename != NULL) g_free(filename);

    save_size_and_position_main_window();

    gtk_main_quit();
}


/*
 * show or hide statusbar
 */
void
on_view_statusbar_menu_toggled(GtkCheckMenuItem * checkmenuitem, gpointer user_data)
{
    gboolean state = gtk_check_menu_item_get_active(checkmenuitem);

    if (state)
		gtk_widget_show(interface.statusbar);
    else
		gtk_widget_hide(interface.statusbar);

    gconf_client_set_bool(client, "/apps/glsof/preferences/show_statusbar", state, NULL);
}


/*
 * make splash window
 */
void make_splash_window(void)
{
    GtkWidget *vbox;
    GdkPixbuf *pixbuf;
    GtkWidget *image;
    GtkWidget *label;
    GdkColor color;

    splash_window = gtk_window_new(GTK_WINDOW_POPUP);
    gtk_window_set_position(GTK_WINDOW(splash_window), GTK_WIN_POS_CENTER);
    gtk_window_set_decorated(GTK_WINDOW(splash_window), FALSE);
    gtk_window_set_resizable(GTK_WINDOW(splash_window), FALSE);
    color.red = 65535;
    color.green = 65535;
    color.blue = 65535;
    gtk_widget_modify_bg(splash_window, GTK_STATE_NORMAL, &color);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(splash_window), vbox);
    gtk_widget_show(vbox);

    gtk_widget_realize(splash_window);
    gdk_window_set_decorations(splash_window->window, 0);

    pixbuf = gdk_pixbuf_new_from_file(PIXMAPS_PATH "glsof-logo.png", NULL);
    image = gtk_image_new_from_pixbuf(pixbuf);
    gtk_container_add(GTK_CONTAINER(vbox), image);
    g_object_unref(pixbuf);
    gtk_widget_show(image);

    label = gtk_label_new(_("Copyleft by Daniele F."));
    gtk_container_add(GTK_CONTAINER(vbox), label);
    gtk_widget_show(label);

    gtk_widget_show(splash_window);

    while (g_main_context_pending(NULL) || !GTK_WIDGET_REALIZED(image))
		gtk_main_iteration();
}


/* 
 * main window
 */
void make_main_window(void)
{
    GladeXML *xml_glade;
    GtkWidget *item_layout2;
    GtkWidget *view_statusbar_menu;
    GtkWidget *debug_output_textview;
    GtkAccelGroup *accel_group;
    GtkTooltips *button_bar_tips;

    button_bar_tips = gtk_tooltips_new();
    accel_group = gtk_accel_group_new();

    xml_glade =	glade_xml_new(GLADE_PATH "glsof_mainwindow.glade", NULL, NULL);
    interface.window = glade_xml_get_widget(xml_glade, "glsof-mainwindow");
    gtk_window_add_accel_group(GTK_WINDOW(interface.window), accel_group);
    interface.vbox = glade_xml_get_widget(xml_glade, "vbox");
    interface.handlebox = glade_xml_get_widget(xml_glade, "bonobodockitem2");
    interface.statusbar = glade_xml_get_widget(xml_glade, "appbar1");
    interface.statusbar_pb = (GtkWidget *) gnome_appbar_get_progress((GnomeAppBar *) interface.statusbar);
    view_statusbar_menu = glade_xml_get_widget(xml_glade, "menu_statusbar1");
    interface.save_as2 = glade_xml_get_widget(xml_glade, "save_output1");
    interface.autorefresh_frame = glade_xml_get_widget(xml_glade, "autorefresh_frame");
    interface.change_settings_sub_menu = glade_xml_get_widget(xml_glade, "change_settings");
    interface.main_window_notebook = glade_xml_get_widget(xml_glade, "main_window_notebook");
    
    /* debug output text buffer */
    debug_output_textview = glade_xml_get_widget(xml_glade, "debug_output_textview");
    interface.debug_button = glade_xml_get_widget(xml_glade, "debug_output_button");
    interface.debug_output_text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (debug_output_textview));
    gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), "heading", "weight", PANGO_WEIGHT_BOLD, "size", 15 * PANGO_SCALE, NULL);
    gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), "red_foreground", "foreground", "red", NULL);
    gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), "bold", "weight", PANGO_WEIGHT_BOLD, NULL);

    /* autorefresh widgets */
    interface.spin1 = glade_xml_get_widget(xml_glade, "spinbutton1");
    interface.spin2 = glade_xml_get_widget(xml_glade, "spinbutton2");
    interface.progressbar = glade_xml_get_widget(xml_glade, "progressbar1");
    interface.start = glade_xml_get_widget(xml_glade, "start_button");
    interface.stop = glade_xml_get_widget(xml_glade, "stop_button");
    interface.autorefresh_button = glade_xml_get_widget(xml_glade, "autorefresh_button");

    /* search */
    interface.hbox_search = glade_xml_get_widget(xml_glade, "hbox_search");
    interface.search_button = glade_xml_get_widget(xml_glade, "search_button");
    interface.search_entry = glade_xml_get_widget(xml_glade, "search_entry");
    interface.combox = glade_xml_get_widget(xml_glade, "combobox");
    gtk_combo_box_set_active(GTK_COMBO_BOX(interface.combox), 0);
    
    /* hpaned */
    interface.hpaned = glade_xml_get_widget(xml_glade, "hpaned_main");

    /* double list layers */
    interface.treeview_left_column = glade_xml_get_widget(xml_glade, "treeview_left_column");
    interface.dlm = double_list_main_new();
    
    get_size_and_position_main_window();
	
    glade_xml_signal_autoconnect(xml_glade);

    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(view_statusbar_menu), gconf_client_get_bool(client, "/apps/glsof/preferences/show_statusbar", NULL));
}
