/*  main - program entry point.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <glade/glade.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf.h>
/*#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>*/
#include <stdio.h>

#include "interface.h"
#include "preferences.h"
#include "double_list_main_object.h"
#include "query_editor.h"

#define CONF_GLOBAL_PREFIX	"/apps/glsof/global"
#define GCONF_PREFERENCES_PATH	"/apps/glsof/preferences/"

/* interface */
extern main_window interface;	/* main window */
extern GtkWidget *splash_window;

/* extern */
GConfClient *client;
gchar *lsof_path = NULL;


static gboolean found_lsof_path(void)
{
    struct stat buf;
    gboolean show_error = FALSE;

    FILE *fp;
    gchar readbuf[BUFSIZ];
    gchar **tmp = NULL;
    guint i = 0, s = 0;

    if ((fp = popen("whereis -b lsof", "r")))
	{
		if (fgets(readbuf, BUFSIZ, fp))	tmp = g_strsplit(readbuf, " ", -1);

		pclose(fp);

		while (readbuf[i])
		{
			if (readbuf[i] == ' ')	s++;
			i++;
		}

		if (s>0)
		{
			int k = 1;

	    		while (k <= s)
			{
				i = 0;

				while (tmp[k][i] != 0)
				{
					if (tmp[k][i] == '\n') tmp[k][i] = 0;
					i++;
				}

				g_stat(tmp[k], &buf);
				if (!g_access(tmp[k], X_OK | F_OK) && !(buf.st_mode & S_IFDIR))
				{
					lsof_path = g_strconcat(tmp[k], " -F +c0 ", NULL);
					gconf_client_set_string(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "lsofpath"), tmp[k], NULL);
					show_error = TRUE;
					break;
				}
				k++;
	    		}
		}
    }

    g_strfreev(tmp);

    return show_error;
}

int main(int argc, char *argv[])
{
    GError *err = NULL;
    gboolean check_show_splash_conf;
    gchar *filename;
    struct stat buf;

#ifdef ENABLE_NLS
    bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);
#endif

    /* init gnome-program */
    gnome_program_init("glsof", VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
    glade_gnome_init();

    /* init gconf */
    gconf_init(argc, argv, NULL);
    client = gconf_client_get_default();

    gconf_client_add_dir(client, CONF_GLOBAL_PREFIX, GCONF_CLIENT_PRELOAD_ONELEVEL, &err);

    if (err)
    {
		g_printerr(_("There was an error loading config from %s. (%s)\n"), CONF_GLOBAL_PREFIX, err->message);
		g_error_free(err);
    }

    /* init lsof's path */
    filename = gconf_client_get_string(client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "lsofpath"), NULL);
    
    if (!g_access(filename, X_OK | F_OK) && !(buf.st_mode & S_IFDIR))
    {
        gconf_client_set_string(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "lsofpath"), filename, NULL);
        lsof_path = g_strconcat(filename, " -F +c0 ", NULL);
    }
    else if (found_lsof_path() == FALSE)
    {
	GtkWidget *dialog;

	dialog = gtk_message_dialog_new(GTK_WINDOW(interface.window), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
					_("Lsof was not found or you have not right permissions.\nPlease, set lsof-path in 'Edit->Preferences'."));

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
    }
    
    check_show_splash_conf = gconf_client_get_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "noshowsplash"), NULL);

    if (check_show_splash_conf == FALSE) make_splash_window();	/* make splash window */
    read_preferences_status_columns();	/* hide & show all columns for double list view */
    make_main_window();	/* make main window */
    
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "refreshall"), NULL) == TRUE)
    	double_list_main_refresh_query_full(interface.dlm);	/* refresh all queries */
    else
        gtk_widget_set_sensitive (interface.hbox_search, FALSE);

    /* close splash window */
    if (check_show_splash_conf == FALSE)
    {
		gtk_widget_destroy(splash_window);
		splash_window = NULL;
    }

    /* show main window and run it */
    gtk_widget_show(interface.window);

    /* loop */
    gtk_main();

    return 0;
}
