/*  popupmenu - Make popupmenu and kill-pid option.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "interface.h"
#include "query_editor.h"
#include "graphics_stats.h"
#include "double_list_main_object.h"

/* interface */
extern main_window interface;	/* main window */

/* query_editor */
extern query_editor pf;


/**************************
 * NEW QUERY POPUP MENU *
 **************************/

/* Make new group of queries popup menu */
static void
view_new_query_popup_menu(GdkEventButton * event)
{
    GtkWidget *menu;
    GtkWidget *menuitem;

    /* popup menu */
    menu = gtk_menu_new();
	
	/* item: New Query */
	menuitem = gtk_menu_item_new_with_label(_("New Query"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	g_signal_connect(menuitem, "activate", (GCallback) make_new_query, NULL);
	
    gtk_widget_show_all(menu);

    /* Note: event can be NULL here when called from view_onPopupMenu;
     *  gdk_event_get_time() accepts a NULL argument */
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, (event != NULL) ? event->button : 0, gdk_event_get_time((GdkEvent *) event));
}

/* Make new query popup menu */
static void
view_read_query_popup_menu(GdkEventButton *event, GtkTreePath *path, guint index)
{
    GtkWidget *menu;
    GtkWidget *menuitem;

    /* popup menu */
    menu = gtk_menu_new();
	
	/* item: Query Editor */
    menuitem = gtk_menu_item_new_with_label(_("Edit query"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	g_signal_connect(menuitem, "activate", (GCallback) on_set_query_editor_clicked, GINT_TO_POINTER(index));
	
	/* item: Rename Query */
	menuitem = gtk_menu_item_new_with_label(_("Rename query"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	g_signal_connect(menuitem, "activate", (GCallback) on_set_rename_query_clicked, path);
	
	/* item: Refresh Query */
	menuitem = gtk_menu_item_new_with_label(_("Refresh query"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	g_signal_connect(menuitem, "activate", (GCallback) on_set_refresh_query_clicked, GINT_TO_POINTER(index));
	
	/* item: Remove Query */
	menuitem = gtk_menu_item_new_with_label(_("Remove query"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	g_signal_connect(menuitem, "activate", (GCallback) on_set_remove_query_clicked, path);
	
	/* item: View Statistics */
	menuitem = gtk_menu_item_new_with_label(_("View statistics"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	if (double_list_main_get_number_processes_for_query (DOUBLE_LIST_MAIN(interface.dlm), index) == 0)
		gtk_widget_set_sensitive(menuitem, FALSE);
	else
		g_signal_connect(menuitem, "activate", (GCallback) on_stats_activate, GINT_TO_POINTER(index));

    gtk_widget_show_all(menu);

    /* Note: event can be NULL here when called from view_onPopupMenu;
     *  gdk_event_get_time() accepts a NULL argument */
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, (event != NULL) ? event->button : 0, gdk_event_get_time((GdkEvent *) event));
}


/*******************
 * MAIN CONTROLLER *
 *******************/
gboolean
view_onButtonPressed(GtkWidget *treeview, GdkEventButton *event, gpointer userdata)
{
	GtkTreePath *path;
    GtkTreeIter iter;
	GtkTreeSelection *selection;
    gboolean selected = TRUE;
	guint index;
	
	
	if(event->type != GDK_BUTTON_PRESS) return FALSE;
    if(event->button != 3) return FALSE;

    if(!gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treeview), event->x, event->y, &path, NULL, NULL, NULL))
		{
			selected = FALSE;
        }
	else
		{
			/* just show context menu if it's on a query name */
			if (gtk_tree_path_get_depth (path) == 1)
			{
				index = gtk_tree_path_get_indices(path)[0];
				
				/* select the row clicked with right button */
				selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
				gtk_tree_selection_select_path (selection, path);
			}
			else
			{
				gtk_tree_path_free(path);
				return TRUE;
			}
        }

	if(selected == TRUE)
		view_read_query_popup_menu(event, path, index);
    else
		view_new_query_popup_menu(event);

    return TRUE;
}
