/*  preferences - Make preferences window and save view-columns-state and other options.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>
#include <stdio.h>

#include "preferences.h"
#include "interface.h"
#include "double_list_main_object.h"

#define GCONF_PREFERENCES_PATH	"/apps/glsof/preferences/"

/* FUNCTIONS PROTOTYPES */
static void make_preferences_window(void);

/* interface */
extern main_window interface;

/* main */
extern gchar *lsof_path;
extern GConfClient *client;

/* extern */
preferences_window preferences;

static const gchar *key[] = { "pid", "pgid", "ppid", "user", "fd", "type", "device", "sizeoff", "nlink", "inode" };

/* signal for 'lsof-infos-button' */
void on_lsof_infos_button_clicked (GtkButton *button, gpointer user_data)
{					
	GladeXML *xml_glade;
	GtkWidget *text_view;
	GtkTextBuffer *buffer;
	GtkTextIter iter;
	FILE *fp;
	gchar **filter_argv;
	gchar *tmp_lsof_path;
	gint err = 2;
	
	static gchar readbuf[BUFSIZ];
	
	xml_glade = glade_xml_new(GLADE_PATH "glsof_about_lsof.glade", NULL, NULL);
	text_view = glade_xml_get_widget(xml_glade, "textview_about_lsof");
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
	
	glade_xml_signal_autoconnect(xml_glade);
	g_object_unref(G_OBJECT(xml_glade));

	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
	
	/* fork 'lsof' */
	tmp_lsof_path = g_strconcat (gconf_client_get_string(client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "lsofpath"), NULL), " -v", NULL);
	filter_argv = g_strsplit (tmp_lsof_path, " ", 0);
	g_free (tmp_lsof_path);
	
	g_spawn_async_with_pipes(NULL, filter_argv, NULL, 0, NULL, NULL, NULL, NULL, NULL, &err, NULL);
	
	fp = fdopen(err, "r");
	
	while (fgets(readbuf, BUFSIZ, fp))
	{
		gtk_text_buffer_insert (buffer, &iter, readbuf, strlen(readbuf));
	}
	
	g_strfreev(filter_argv);
	fclose(fp);
}

/* signal for 'open-button' */
void on_lsof_path_button_clicked (GtkButton *button, gpointer user_data)
{
	struct stat buf;
	gchar *filename;
	GtkWidget *dialog;
	GtkFileFilter *filter;
	
	dialog = gtk_file_chooser_dialog_new ("Find lsof binary...",
					      preferences.window,
					      GTK_FILE_CHOOSER_ACTION_OPEN,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					      NULL);
					      
	filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "lsof");
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);

	while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
  	{
    		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    		
    		g_stat(filename, &buf);
		if (!g_access(filename, X_OK | F_OK) && !(buf.st_mode & S_IFDIR))
		{
			if (lsof_path != NULL) g_free (lsof_path);
			lsof_path = g_strconcat(filename, " -F +c0 ", NULL);
			
			gconf_client_set_string(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "lsofpath"), filename, NULL);
			gtk_entry_set_text (preferences.sets_entry_conf, filename);
			gtk_widget_set_sensitive (preferences.lsof_infos_button, TRUE);
			
			g_free (filename);
			break;
		}
		else
		{
			GtkWidget *dialog;
			
			dialog = gtk_message_dialog_new(GTK_WINDOW(interface.window), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
						_("You have not right permissions to run Lsof.\nPlease, check Lsof's permissions."));
						
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
    		
    		g_free (filename);
  	}

	gtk_widget_destroy (dialog);
}

/* read preferences */
void read_preferences_status_columns (void)
{
	guint i;
	
	for (i = 0; i < 10; i++)
		preferences.state_fields[i] = gconf_client_get_bool(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,key[i]), NULL);
}

/* columns hide & view */
void show_hide_columns(GtkToggleButton * togglebutton, gpointer user_data)
{
    guint i, j, number_tables;

    i = GPOINTER_TO_INT(user_data);

    preferences.state_fields[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton));
    gconf_client_set_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, key[i]), preferences.state_fields[i], NULL);

    /* double list main */
    double_list_main_set_status_columns (DOUBLE_LIST_MAIN(interface.dlm), i);
}


void save_size_and_position_main_window(void)
{
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,"keepwindowsize"),NULL))
	{
		gint x, y, w, h, hpaned_size;
		
		gtk_window_get_position(GTK_WINDOW(interface.window), &x, &y);
		gtk_window_get_size(GTK_WINDOW(interface.window), &w, &h);
		hpaned_size = gtk_paned_get_position(GTK_PANED(interface.hpaned));
		
		gconf_client_set_int(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "windowpositionx"), x, NULL);
		gconf_client_set_int(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "windowpositiony"), y, NULL);
		gconf_client_set_int(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "window-width"), w, NULL);
		gconf_client_set_int(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "window-height"), h, NULL);
		gconf_client_set_int(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "hpaned_main_size"), hpaned_size, NULL);
    }
}


void get_size_and_position_main_window(void)
{
    gint x, y, w, h, hpaned_size;

    x = gconf_client_get_int(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "windowpositionx"), NULL);
    y = gconf_client_get_int(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "windowpositiony"), NULL);
    w = gconf_client_get_int(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "window-width"), NULL);
    h = gconf_client_get_int(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "window-height"), NULL);
    hpaned_size = gconf_client_get_int(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "hpaned_main_size"), NULL);

    gtk_window_move(GTK_WINDOW(interface.window), x, y);
    gtk_window_resize(GTK_WINDOW(interface.window), w, h);
    gtk_paned_set_position(GTK_PANED(interface.hpaned), hpaned_size);
}


/* Configuration Window */
void
on_main_cb1_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
    gconf_client_set_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH, "keepwindowsize"),gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(preferences.sets_check_conf)),NULL);
}

void
on_main_cb2_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
    gconf_client_set_bool(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,"noshowsplash"),gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(preferences.sets_check_show_splash_conf)),NULL);
}

void
on_main_cb3_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
    gconf_client_set_bool(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,"refreshall"),gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(preferences.sets_check_refresh_all)),NULL);
}

void on_preferences_clicked(GtkButton * button, gpointer user_data)
{
    make_preferences_window();
}

static void make_preferences_window(void)
{
    GladeXML *xml_glade;
    gchar *filepath;
    guint i;

    xml_glade =	glade_xml_new(GLADE_PATH "glsof_preferences.glade", NULL, NULL);

    preferences.window = glade_xml_get_widget(xml_glade, "glsof-preferences");
    preferences.sets_check_conf = glade_xml_get_widget(xml_glade, "main_cb1");
    preferences.sets_check_show_splash_conf = glade_xml_get_widget(xml_glade, "main_cb2");
    preferences.sets_check_refresh_all = glade_xml_get_widget(xml_glade, "main_cb3");

    preferences.checkbutton_fields[0] =	glade_xml_get_widget(xml_glade, "checkbutton1");
    preferences.checkbutton_fields[1] =	glade_xml_get_widget(xml_glade, "checkbutton2");
    preferences.checkbutton_fields[2] =	glade_xml_get_widget(xml_glade, "checkbutton3");
    preferences.checkbutton_fields[3] =	glade_xml_get_widget(xml_glade, "checkbutton4");
    preferences.checkbutton_fields[4] =	glade_xml_get_widget(xml_glade, "checkbutton5");
    preferences.checkbutton_fields[5] =	glade_xml_get_widget(xml_glade, "checkbutton6");
    preferences.checkbutton_fields[6] =	glade_xml_get_widget(xml_glade, "checkbutton7");
    preferences.checkbutton_fields[7] =	glade_xml_get_widget(xml_glade, "checkbutton8");
    preferences.checkbutton_fields[8] =	glade_xml_get_widget(xml_glade, "checkbutton9");
    preferences.checkbutton_fields[9] =	glade_xml_get_widget(xml_glade, "checkbutton10");
    
    /* lsof path */
    preferences.lsof_infos_button = glade_xml_get_widget(xml_glade, "lsof_infos_button");
    preferences.sets_entry_conf = glade_xml_get_widget(xml_glade, "lsof_path_entry");
    filepath = gconf_client_get_string(client, gconf_concat_dir_and_key (GCONF_PREFERENCES_PATH, "lsofpath"), NULL);
    gtk_entry_set_text (preferences.sets_entry_conf, filepath);
    
    if (strlen(filepath) > 0)
    	gtk_widget_set_sensitive (preferences.lsof_infos_button, TRUE);

    g_free(filepath);
    
    /* signals */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preferences.sets_check_conf), \
    	gconf_client_get_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,"keepwindowsize"),NULL));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preferences.sets_check_show_splash_conf), \
    	gconf_client_get_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,"noshowsplash"),NULL));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preferences.sets_check_refresh_all), \
    	gconf_client_get_bool(client, gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,"refreshall"),NULL));

    for (i = 0; i < 10; i++)
	{
		preferences.state_fields[i] = gconf_client_get_bool(client,gconf_concat_dir_and_key(GCONF_PREFERENCES_PATH,key[i]), NULL);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preferences.checkbutton_fields[i]),preferences.state_fields[i]);
		
		g_signal_connect((gpointer) preferences.checkbutton_fields[i],"clicked", G_CALLBACK(show_hide_columns),GINT_TO_POINTER(i));
    }

    glade_xml_signal_autoconnect(xml_glade);
    g_object_unref(G_OBJECT(xml_glade));
}
