/*  query_editor - Make window for query editing.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */

#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>
#include <string.h>

#include "query_editor.h"
#include "double_list_main_object.h"
#include "interface.h"

/* FUNCTIONS PROTOTYPES */
static void selected_row_central_list (GtkTreeSelection *, gpointer);
static void on_cb_active_toggled (GtkToggleButton *, gpointer);
static void selection_changed (GtkTreeSelection *, gpointer);
static gboolean check_empty_list (void);
static void free_all_query_lists (void);
void on_timeout_toggled(GtkToggleButton *, gpointer);
void on_nlinks_toggled(GtkToggleButton *, gpointer);
void on_add_button_clicked (GtkButton *, gpointer);
static void insert_text_handler_numeric (GtkEntry *, const gchar *, gint, gint *, gpointer);
static void insert_text_handler_alphanumeric_and_space (GtkEntry *, const gchar *, gint, gint *, gpointer);
static gint on_accept_dialog (GtkWidget *);
static gint check_new_input1 (GtkWidget *);
static gint check_new_input2 (GtkWidget *);
static gint check_new_input3 (GtkWidget *);
void on_modify_button_clicked (GtkButton *, gpointer);
static void modify_fill_dialog (void);
void on_remove_button_clicked (GtkButton *, gpointer);
static void load_query (void);
static void save_query (void);
static void copy_query (const gchar *, const gchar *);
static void remove_query (void);

/* main */
extern GConfClient *client;
extern gchar *lsof_path;

/* interface */
extern main_window interface;

/* extern */
query_editor pf;

static const gchar *titles_dialog [] = { _("Processes"), _("Network"), _("ID/Login Name"), _("File Descriptors"), _("Directories"), _("PID"), _("GPID") };
			
			
/*******************************************************************************
 *                                                                             *
 *                               QUERY-WINDOW                                  *
 *                                                                             *
 *******************************************************************************/

void make_query_editor (gboolean new_query)
{
    GladeXML *xml_glade;
    GtkTreeIter iter, iter_first_line;
    GtkWidget *scrolledwindow;
    GtkListStore *model;
    GtkCellRenderer *cell;
    GtkWidget *view;
    GtkTreeSelection *selection;
	gchar *title;

    xml_glade =	glade_xml_new(GLADE_PATH "glsof_query_editor.glade", NULL, NULL);
    pf.window = glade_xml_get_widget(xml_glade, "query_editor");
	
	/* set title */
	title = g_strconcat (_("Query: "), pf.gconf_query_basename, NULL);
	gtk_window_set_title (pf.window, title);
	g_free (title);

    /*
     * 'Main' from tab menu
     */

    pf.selected = 0;	/* first list is selected */
	pf.modify_dialog_active = FALSE;

    pf.add_button = glade_xml_get_widget(xml_glade, "add_button");
    pf.modify_button = glade_xml_get_widget(xml_glade, "modify_button");
    pf.remove_button = glade_xml_get_widget(xml_glade, "remove_button");

    pf.notebook_lists_container = glade_xml_get_widget(xml_glade, "notebook_lists_container");

    pf.cb_active[PROCESSES] = glade_xml_get_widget(xml_glade, "cb_active_processes");
    pf.cb_active[NETWORK] = glade_xml_get_widget(xml_glade, "cb_active_network");
    pf.cb_active[IDLN] = glade_xml_get_widget(xml_glade, "cb_active_idln");
    pf.cb_active[FD] = glade_xml_get_widget(xml_glade, "cb_active_fd");
    pf.cb_active[DIR] = glade_xml_get_widget(xml_glade, "cb_active_dir");
    pf.cb_active[PID] = glade_xml_get_widget(xml_glade, "cb_active_pid");
    pf.cb_active[GPID] = glade_xml_get_widget(xml_glade, "cb_active_gpid");
	
	pf.cb_exclude_fd = glade_xml_get_widget(xml_glade, "cb_exclude_fd");

    /***************  MAKE ALL LISTS ***************/

    /****************** PROCESSES ******************/

    pf.liststore[PROCESSES] = gtk_list_store_new (1, G_TYPE_STRING);

    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pf.liststore[PROCESSES]));

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1, _("Process"), cell,"text", 0, NULL);

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);

    g_object_unref(pf.liststore[PROCESSES]);
    pf.scrolledwindow[PROCESSES] = glade_xml_get_widget(xml_glade, "scrolledwindow1");
    gtk_container_add (GTK_CONTAINER (pf.scrolledwindow[PROCESSES]), view);

    g_signal_connect(G_OBJECT(selection),"changed",G_CALLBACK(selected_row_central_list), NULL);
    g_signal_connect(G_OBJECT(pf.cb_active[PROCESSES]), "toggled", G_CALLBACK(on_cb_active_toggled), selection);

    /****************** NETWORK ******************/

    pf.liststore[NETWORK] = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pf.liststore[NETWORK]));

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1, _("Address"), cell,"text", 0, NULL);

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1, _("Port"), cell,"text", 1, NULL);

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1, _("IPV"), cell,"text", 2, NULL);

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1, _("Protocol"), cell,"text", 3, NULL);

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);

    g_object_unref(pf.liststore[NETWORK]);
    pf.scrolledwindow[NETWORK] = glade_xml_get_widget(xml_glade, "scrolledwindow2");
    gtk_container_add (GTK_CONTAINER (pf.scrolledwindow[NETWORK]), view);

    g_signal_connect(G_OBJECT(selection),"changed",G_CALLBACK(selected_row_central_list), NULL);
    g_signal_connect(G_OBJECT(pf.cb_active[NETWORK]), "toggled", G_CALLBACK(on_cb_active_toggled), selection);

    /****************** ID/LOGIN NAME ******************/

    pf.liststore[IDLN] = gtk_list_store_new (1, G_TYPE_STRING);

    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pf.liststore[IDLN]));

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1, _("ID/Login Name"), cell,"text", 0, NULL);

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);

    g_object_unref(pf.liststore[IDLN]);
    pf.scrolledwindow[IDLN] = glade_xml_get_widget(xml_glade, "scrolledwindow3");
    gtk_container_add (GTK_CONTAINER (pf.scrolledwindow[IDLN]), view);

    g_signal_connect(G_OBJECT(selection),"changed",G_CALLBACK(selected_row_central_list), NULL);
    g_signal_connect(G_OBJECT(pf.cb_active[IDLN]), "toggled", G_CALLBACK(on_cb_active_toggled), selection);

    /****************** FILE DESCRIPTOR ******************/

    pf.liststore[FD] = gtk_list_store_new (1, G_TYPE_STRING);

    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pf.liststore[FD]));

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1,_("File Descriptor"),cell,"text",0, NULL);

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);

    g_object_unref(pf.liststore[FD]);
    pf.scrolledwindow[FD] = glade_xml_get_widget(xml_glade, "scrolledwindow4");
    gtk_container_add (GTK_CONTAINER (pf.scrolledwindow[FD]), view);

    g_signal_connect(G_OBJECT(selection),"changed",G_CALLBACK(selected_row_central_list), NULL);
    g_signal_connect(G_OBJECT(pf.cb_active[FD]), "toggled", G_CALLBACK(on_cb_active_toggled), selection);

    /****************** DIRECTORY ******************/

    pf.liststore[DIR] = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pf.liststore[DIR]));

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1,_("Directory"),cell,"text",0, NULL);

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1,_("Full-descent directory tree"),cell,"text",1, NULL);

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);

    g_object_unref(pf.liststore[DIR]);
    pf.scrolledwindow[DIR] = glade_xml_get_widget(xml_glade, "scrolledwindow5");
    gtk_container_add (GTK_CONTAINER (pf.scrolledwindow[DIR]), view);

    g_signal_connect(G_OBJECT(selection),"changed",G_CALLBACK(selected_row_central_list), NULL);
    g_signal_connect(G_OBJECT(pf.cb_active[DIR]), "toggled", G_CALLBACK(on_cb_active_toggled), selection);

    /****************** PID ******************/

    pf.liststore[PID] = gtk_list_store_new (1, G_TYPE_STRING);

    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pf.liststore[PID]));

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1,_("PID"),cell,"text",0, NULL);

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);

    g_object_unref(pf.liststore[PID]);
    pf.scrolledwindow[PID] = glade_xml_get_widget(xml_glade, "scrolledwindow6");
    gtk_container_add (GTK_CONTAINER (pf.scrolledwindow[PID]), view);

    g_signal_connect(G_OBJECT(selection),"changed",G_CALLBACK(selected_row_central_list), NULL);
    g_signal_connect(G_OBJECT(pf.cb_active[PID]), "toggled", G_CALLBACK(on_cb_active_toggled), selection);

    /****************** GPID ******************/

    pf.liststore[GPID] = gtk_list_store_new (1, G_TYPE_STRING);

    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pf.liststore[GPID]));

    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),-1,_("GPID"),cell,"text",0, NULL);

    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);

    g_object_unref(pf.liststore[GPID]);
    pf.scrolledwindow[GPID] = glade_xml_get_widget(xml_glade, "scrolledwindow7");
    gtk_container_add (GTK_CONTAINER (pf.scrolledwindow[GPID]), view);

    g_signal_connect(G_OBJECT(selection),"changed",G_CALLBACK(selected_row_central_list), NULL);
    g_signal_connect(G_OBJECT(pf.cb_active[GPID]), "toggled", G_CALLBACK(on_cb_active_toggled), selection);

    /**************** END LISTS ****************/

    /****************** MENU ******************/

    /* make control list */
    model = gtk_list_store_new (1, G_TYPE_STRING);
    /* set names in menu list */
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, 0, _("Processes"), -1);
    iter_first_line = iter;
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, 0, _("Network"), -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, 0, _("ID/Login Name"), -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, 0, _("File Descriptors"), -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, 0, _("Directories"), -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, 0, _("PID"), -1);
    gtk_list_store_append (model, &iter);
    gtk_list_store_set (model, &iter, 0, _("GPID"), -1);

    /* completing control list */
    view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(model));
    cell = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view), -1, _("   Menu  "), cell, "text", 0, NULL);
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(view), FALSE);
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(view));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    gtk_widget_show(view);
    g_object_unref(model);

    scrolledwindow = glade_xml_get_widget(xml_glade, "scrolledwindow_menu");
    gtk_container_add (GTK_CONTAINER (scrolledwindow), view);

    /* signals */
    g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(selection_changed), NULL);

    /* select menu first line and so first table */
    gtk_tree_selection_select_iter (selection, &iter_first_line);

    /* 
     * 'Others' from tab menu
     */

    /* general - id or login name */
    pf.id_radio = glade_xml_get_widget(xml_glade, "id_radio_button");
    pf.login_radio = glade_xml_get_widget(xml_glade, "login_radio_button");

    /* general - offset or size */
    pf.offset_radio = glade_xml_get_widget(xml_glade, "file_offset_radio_button");
    pf.size_radio = glade_xml_get_widget(xml_glade, "file_size_radio_button");

    /* general - AND all settings */
    pf.and_check = glade_xml_get_widget(xml_glade, "and_settings_check_button");

    /* general - avoid "glsof" and "lsof" in output */
    pf.hideglsof_check = glade_xml_get_widget(xml_glade, "no_output_glsof_lsof_check_button");

    /* network - ip-form */
    pf.ipform_check = glade_xml_get_widget(xml_glade, "ip_form_check_button");

    /* network - nfs */
    pf.nfs_check = glade_xml_get_widget(xml_glade, "nfs_check_button");

    /* network - port numbers */
    pf.portnumbers_check = glade_xml_get_widget(xml_glade, "port_number_check_button");

    /* network - unix sockets */
    pf.unixsockets_check = glade_xml_get_widget(xml_glade, "unix_sockets_check_button");

    /* kernel - timeout seconds */
    pf.timeout_check = glade_xml_get_widget(xml_glade, "timeout_check_button");
    pf.timeout_entry = glade_xml_get_widget(xml_glade, "timeout_spin_button");

    /* kernel - avoid */
    pf.avoid_check = glade_xml_get_widget(xml_glade, "avoid_check_button");
    
    /* processes - max number of links for a file */
    pf.nlinks_check = glade_xml_get_widget(xml_glade, "max_links_check_button");
    pf.nlinks_entry = glade_xml_get_widget(xml_glade, "max_links_spin_button");

    /***************************/
    /* active signals and exit */
    /***************************/
    glade_xml_signal_autoconnect(xml_glade);
    g_object_unref(G_OBJECT(xml_glade));
    
    /* load query settings */
	if (new_query == FALSE) load_query();
	else save_query();
}


/*******************************************************************************
 *                                                                             *
 *                   SIGNALS AND FUNCTIONS FOR QUERY-WINDOW                    *
 *                                                                             *
 *******************************************************************************/


/*
 * ok button query editor
 */
void on_ok_button_clicked (GtkButton * button, gpointer user_data)
{
	if (check_empty_list() == FALSE)
	{
		save_query();
		free_all_query_lists();
		gtk_widget_destroy(pf.window);
	}
}

/*
 * cancel button query editor
 */
void on_cancel_button_clicked (GtkButton * button, gpointer user_data)
{
	free_all_query_lists();
	gtk_widget_destroy(pf.window);
}

/*
 * active-disactive "modify" and "remove" buttons when a central list row is selected
 */
static void selected_row_central_list (GtkTreeSelection *treeselection, gpointer user_data)
{
    gboolean state;

    state = gtk_tree_selection_get_selected (treeselection, NULL, &(pf.iter_selected_row));

    gtk_widget_set_sensitive(pf.modify_button, state);
    gtk_widget_set_sensitive(pf.remove_button, state);
}

/*
 * check if single list is empty, network apart
 */
static gboolean check_empty_list (void)
{
	guint i;
	/* no strings for PROCESSES and NETWORK */
	static const gchar *warnings[] = {NULL, NULL, \
		_("\"ID/Login name\" list is selected but empty."), _("\"File Descriptors\" list is selected but empty."), \
		_("\"Directories\" list is selected but empty."), _("\"PID\" list is selected but empty."), \
		_("\"GPID\" list is selected but empty.") };
	
	if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[PROCESSES])) && (g_slist_length (pf.list[PROCESSES]) == 0) )
	{
		WARNING_DIALOG (_("\"Processes\" list is selected but empty."), pf.window);
		return TRUE;
	}
	/* do not control for NETWORK */
	for (i=IDLN; i<MENUITEMS; i++)
	{
		if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[i])) && (g_slist_length (pf.list[i]) == 0) )
		{
			WARNING_DIALOG (warnings[i], pf.window);
			return TRUE;
		}
	}
	
	return FALSE;
}


/*
 * show or hide central lists and active-disactive side-buttons
 */
static void on_cb_active_toggled (GtkToggleButton *button, gpointer user_data)
{
	gboolean state;
	
    gtk_tree_selection_unselect_all (GTK_TREE_SELECTION(user_data));
    
	state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
	gtk_widget_set_sensitive (pf.scrolledwindow[pf.selected], state);
    gtk_widget_set_sensitive (pf.add_button, state);
	
	if (pf.selected == FD)
		gtk_widget_set_sensitive (pf.cb_exclude_fd, state);
}


/*
 * select list from menu
 */
static void selection_changed (GtkTreeSelection * selection, gpointer data)
{
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
	GtkTreePath *path;
	gint position;
	/* next variables are useful to know if a row is selected
	   in a central list, and set sensitive for "modify" and
	   "remove" buttons. */
	GtkTreeView *treeview;
	GtkTreeSelection *sel;
	gboolean state;


	path = gtk_tree_model_get_path(model, &iter);
	pf.selected = gtk_tree_path_get_indices(path)[0];

	/* show list and sensitive add_button */
	state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pf.cb_active[pf.selected]));
	gtk_notebook_set_current_page (GTK_NOTEBOOK(pf.notebook_lists_container), pf.selected);
	gtk_widget_set_sensitive (pf.add_button, state);

	gtk_tree_path_free(path);

	/* ugly but working :)*/
	/* this code gets a central list and controls if a row is selected.
	   If so, set sensitive of "modify" and "remove" buttons appropriately */
	treeview = GTK_TREE_VIEW(gtk_bin_get_child (GTK_BIN(pf.scrolledwindow[pf.selected])));
	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	state = gtk_tree_selection_get_selected (sel, NULL, NULL);
	gtk_widget_set_sensitive (pf.modify_button, state);
	gtk_widget_set_sensitive (pf.remove_button, state);
    }
}

static void free_all_query_lists (void) {
	
	if (pf.gconf_query_basename != NULL)
	{
		GSList *iter_list = NULL;
		guint i;
		dir_data *dd;
		network_data *nd;

		/* free all lists */
		FREE_LIST(pf.list[PROCESSES]);
		FREE_LIST(pf.list[IDLN]);
		FREE_LIST(pf.list[FD]);
		FREE_LIST(pf.list[PID]);
		FREE_LIST(pf.list[GPID]);
	
		iter_list = pf.list[DIR];
		while(iter_list != NULL)
		{
			dd = iter_list->data;
			g_free (dd->path);
			g_free (dd);
			iter_list = g_slist_next(iter_list);
		}
		g_slist_free (pf.list[DIR]);
		pf.list[DIR] = NULL;
	
		iter_list = pf.list[NETWORK];
		while(iter_list != NULL)
		{
			nd = iter_list->data;
			g_free (nd->host);
			g_free (nd->port);
			g_free (nd->ipv);
			g_free (nd->protocol);
			g_free (nd);
			iter_list = g_slist_next(iter_list);
		}
		g_slist_free (pf.list[NETWORK]);
		pf.list[NETWORK] = NULL;	
	
		g_free (pf.gconf_query_basename);
		pf.gconf_query_basename = NULL;
	}
}

void on_timeout_toggled(GtkToggleButton * button, gpointer user_data)
{
	gtk_widget_set_sensitive(pf.timeout_entry, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
}

void on_nlinks_toggled(GtkToggleButton * button, gpointer user_data)
{
	gtk_widget_set_sensitive(pf.nlinks_entry, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
}


/*******************************************************************************
 *                                                                             *
 *           SIGNALS AND FUNCTIONS FOR QUERY-WINDOW FROM POPUP MENU            *
 *                                                                             *
 *******************************************************************************/

static gchar *query_name_input (gchar *title)
{
	gchar *name=NULL;
	GtkWidget *hbox, *label;
	gint value;
	GtkWidget *dialog = gtk_dialog_new_with_buttons (title,
                                                  GTK_WINDOW(interface.window),
                                                  GTK_DIALOG_MODAL,
                                                  GTK_STOCK_CANCEL,
                                                  GTK_RESPONSE_REJECT,
						  						  GTK_STOCK_OK,
                                                  GTK_RESPONSE_ACCEPT,
                                                  NULL);
	
	ADD_DIALOG (_("Name:"));
	if (strcmp(title, _("New Query"))==0)
		g_signal_connect(G_OBJECT(pf.entry1), "insert_text", G_CALLBACK(insert_text_handler_alphanumeric_and_space), NULL);
	
	gtk_widget_show_all (dialog);
		
	while ( ( value = gtk_dialog_run (GTK_DIALOG (dialog)) ) == GTK_RESPONSE_ACCEPT )
	{
		name = g_strdup (gtk_entry_get_text (GTK_ENTRY(pf.entry1)));
		if (strlen (name) == 0)
		{
			WARNING_DIALOG (_("Please, input a name."), dialog);
		}
		else if (double_list_main_add_query_check (DOUBLE_LIST_MAIN(interface.dlm), name) == TRUE) break;
		else
		{
			WARNING_DIALOG (_("This query name is already included."), dialog);
		}
	}
	
	if (value == GTK_RESPONSE_REJECT)
	{
		g_free (name);
		name = NULL;
	}

	gtk_widget_destroy (dialog);

	return name;
}

/* From popmenu.c:
 * make new query editor window
 */
void make_new_query (void)
{
	gchar *name;
	
	if ( (name = query_name_input (_("New Query"))) == NULL )
		return;
	
	double_list_main_add_query (interface.dlm, name);
	
	if (pf.gconf_query_basename != NULL) g_free (pf.gconf_query_basename);
	pf.gconf_query_basename = name;
	make_query_editor (TRUE);
}

/* From popmenu.c:
 * make query editor window
 */
void on_set_query_editor_clicked (GtkButton * button, gpointer user_data)
{
	gchar *name_query;
	DoubleListMain *dlm;
	
	dlm = interface.dlm;
	
	name_query = g_slist_nth_data (dlm->queries, GPOINTER_TO_INT (user_data));
	
	if (pf.gconf_query_basename != NULL) g_free (pf.gconf_query_basename);
	pf.gconf_query_basename = g_strdup (name_query);
	
	make_query_editor(FALSE);
}

/* From popmenu.c:
 * refresh query
 */
void on_set_refresh_query_clicked (GtkButton * button, gpointer user_data)
{
	gboolean state_search;
	DoubleListMain *dlm;
	gchar *prompt;
	GtkTextIter start, end;
	
	if (lsof_path == NULL)
	{
		GtkWidget *dialog;
		
		dialog = gtk_message_dialog_new(GTK_WINDOW(interface.window), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
						_("Lsof was not found.\nPlease, set lsof-path in 'Edit->Preferences'."));

		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		
		return;
	}
	
	/* No sensitive when refresh query */
	gtk_widget_set_sensitive(interface.vbox, FALSE);
	
	/* clear debug output text view */
	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER(interface.debug_output_text_buffer),&start, &end);
	gtk_text_buffer_delete (GTK_TEXT_BUFFER(interface.debug_output_text_buffer),&start, &end);
	
	/* start refresh */
	dlm = DOUBLE_LIST_MAIN (interface.dlm);
	double_list_main_refresh_query (dlm, GPOINTER_TO_INT (user_data));
	
	/* search sensitive */
	state_search = double_list_main_is_there_a_row (dlm);
	gtk_widget_set_sensitive (interface.hbox_search, state_search);
	
	/* show message on status bar */
	prompt = g_strconcat (pf.gconf_query_basename, _(" query refreshed."), NULL);
	gnome_appbar_pop(GNOME_APPBAR(interface.statusbar));
	gnome_appbar_push(GNOME_APPBAR(interface.statusbar), prompt);
	g_free (prompt);
	
	/* Main Window Sensitive */
	gtk_widget_grab_focus(interface.treeview_left_column);
	gtk_widget_set_sensitive(interface.vbox, TRUE);
}

/* From popmenu.c:
 * rename query
 */
void on_set_rename_query_clicked (GtkButton * button, gpointer user_data)
{
	gchar *old_name, *bold_new_name, *new_name = NULL;
	GtkTreeIter iter;
	guint index;
	DoubleListMain *dlm;
	GSList *list;
	
	if ( (new_name = query_name_input (_("New Query's name"))) == NULL )
		return;
	
	dlm = interface.dlm;
	index = gtk_tree_path_get_indices(user_data)[0];
	old_name = g_slist_nth_data (dlm->queries, index);
	
	bold_new_name = g_strconcat ("<b>",new_name,"</b>",NULL);
	gtk_tree_model_get_iter(GTK_TREE_MODEL(dlm->treestore), &iter, user_data);
	gtk_tree_store_set(dlm->treestore, &iter, 1, bold_new_name, -1);
	g_free (bold_new_name);
	
	copy_query (old_name, new_name);
	
	/* ...and remove old query */
	if (pf.gconf_query_basename != NULL) g_free (pf.gconf_query_basename);
	pf.gconf_query_basename = g_strdup (old_name);
	remove_query();
	
	/* modify list queries */
	list = g_slist_nth (dlm->queries, index);
	list->data = new_name;
	g_free (old_name);
	
	/* save queries list */
	gconf_client_set_list(client, gconf_concat_dir_and_key(GCONF_PATH_GLOBAL, "list_queries"), GCONF_VALUE_STRING, dlm->queries, NULL);
}

/* From popmenu.c:
 * remove query
 */
void on_set_remove_query_clicked (GtkButton * button, gpointer user_data)
{
	double_list_main_remove_query (interface.dlm, user_data, gtk_tree_path_get_indices(user_data)[0]);
	remove_query ();
}


/*******************************************************************************
 *                                                                             *
 *                        ENTRY-TEXT FILTER FOR DIALOGS                        *
 *                                                                             *
 *******************************************************************************/

/*
 * show only numeric characters in GtkEntry
 */
static void insert_text_handler_numeric (GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
  GtkEditable *editable = GTK_EDITABLE(entry);
  int i, count=0;
  gchar *result = g_new (gchar, length);

  for (i=0; i < length; i++) {
    if (!g_ascii_isdigit(text[i]))
      continue;
    result[count++] = text[i];
  }
  
  if (count > 0)
  {
	  g_signal_handlers_block_by_func (G_OBJECT (editable), G_CALLBACK (insert_text_handler_numeric), data);
      gtk_editable_insert_text (editable, result, count, position);
      g_signal_handlers_unblock_by_func (G_OBJECT (editable), G_CALLBACK (insert_text_handler_numeric), data);
  }
  g_signal_stop_emission_by_name (G_OBJECT (editable), "insert_text");

  g_free (result);
}

/*
 * show only alphanumeric and ' ' characters in GtkEntry
 */
static void insert_text_handler_alphanumeric_and_space (GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
  GtkEditable *editable = GTK_EDITABLE(entry);
  int i, count=0;
  gchar *result = g_new (gchar, length);

  for (i=0; i < length; i++) {
    if ((!g_ascii_isalnum(text[i])) && (text[i]!=' '))
      continue;
    result[count++] = text[i];
  }
  
  if (count > 0)
  {
	  g_signal_handlers_block_by_func (G_OBJECT (editable), G_CALLBACK (insert_text_handler_alphanumeric_and_space), data);
      gtk_editable_insert_text (editable, result, count, position);
      g_signal_handlers_unblock_by_func (G_OBJECT (editable), G_CALLBACK (insert_text_handler_alphanumeric_and_space), data);
  }
  g_signal_stop_emission_by_name (G_OBJECT (editable), "insert_text");

  g_free (result);
}


/*******************************************************************************
 *                                                                             *
 *                       ADD BUTTON SIGNAL & FUNCTIONS                         *
 *                                                                             *
 *******************************************************************************/


/*
 * this function is used for "modify" button too,
 * to make and fill entries, set check buttons, etc.
 */
void on_add_button_clicked (GtkButton *button, gpointer user_data)
{
    guint selected;
    GtkWidget *hbox, *table, *label;
	
	selected = pf.selected;

    GtkWidget *dialog = gtk_dialog_new_with_buttons (titles_dialog[selected],
                                                  GTK_WINDOW(pf.window),
                                                  GTK_DIALOG_MODAL,
                                                  GTK_STOCK_CANCEL,
                                                  GTK_RESPONSE_REJECT,
						  						  GTK_STOCK_OK,
                                                  GTK_RESPONSE_ACCEPT,
                                                  NULL);

    /*
	 * make the single dialog:
	 * common for "modify" and "add" control
	 */
    switch (selected)
    {
	case PROCESSES:
		ADD_DIALOG(_("Process:"));
		break;

	case NETWORK:
		table = gtk_table_new (4, 2, FALSE);
		gtk_table_set_col_spacings (GTK_TABLE(table), 6);
		gtk_table_set_row_spacings (GTK_TABLE(table), 6);
		gtk_container_set_border_width (GTK_CONTAINER (table),12);
		label = gtk_label_new (_("Host name or IP:"));
		gtk_misc_set_alignment (GTK_MISC(label), 1, 0.5);
		gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,0,1);
		label = gtk_label_new (_("List of ports/services:"));
		gtk_misc_set_alignment (GTK_MISC(label), 1, 0.5);
		gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,1,2);
		label = gtk_label_new (_("IPV:"));
		gtk_misc_set_alignment (GTK_MISC(label), 1, 0.5);
		gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,2,3);
		label = gtk_label_new (_("Protocol:"));
		gtk_misc_set_alignment (GTK_MISC(label), 1, 0.5);
		gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,3,4);
		pf.entry1 = gtk_entry_new ();
		gtk_table_attach_defaults (GTK_TABLE(table), pf.entry1, 1,2,0,1);
		pf.entry2 = gtk_entry_new ();
		gtk_table_attach_defaults (GTK_TABLE(table), pf.entry2, 1,2,1,2);
		pf.combobox1 = gtk_combo_box_new_text ();
		gtk_combo_box_append_text (GTK_COMBO_BOX (pf.combobox1), "");
		gtk_combo_box_append_text (GTK_COMBO_BOX (pf.combobox1), _("4"));
		gtk_combo_box_append_text (GTK_COMBO_BOX (pf.combobox1), _("6"));
		gtk_combo_box_set_active (GTK_COMBO_BOX (pf.combobox1), 0);
		gtk_table_attach_defaults (GTK_TABLE(table), pf.combobox1, 1,2,2,3);
		pf.combobox2 = gtk_combo_box_new_text();
		gtk_combo_box_append_text (GTK_COMBO_BOX (pf.combobox2), "");
		gtk_combo_box_append_text (GTK_COMBO_BOX (pf.combobox2), _("TCP"));
		gtk_combo_box_append_text (GTK_COMBO_BOX (pf.combobox2), _("UDP"));
		gtk_combo_box_set_active (GTK_COMBO_BOX (pf.combobox2), 0);
		gtk_table_attach_defaults (GTK_TABLE(table), pf.combobox2, 1,2,3,4);
		gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);
		break;

	case IDLN:
		ADD_DIALOG(_("ID or Login name:"));
		break;

	case FD:
		ADD_DIALOG(_("File descriptor:"));
		break;

	case DIR:
		hbox = gtk_hbox_new (FALSE, 6);
		gtk_container_set_border_width (GTK_CONTAINER (hbox),12);
		label = gtk_label_new (_("Directory:"));
		pf.entry1 = gtk_entry_new ();
		gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_box_pack_start (GTK_BOX(hbox), pf.entry1, FALSE, FALSE, 0);
		pf.check1 = gtk_check_button_new_with_label (_("Full-descent directory tree"));
		gtk_box_pack_start (GTK_BOX(hbox), pf.check1, FALSE, FALSE, 0);
		gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 0);
		break;

	case PID:
		ADD_DIALOG(_("PID:"));
		g_signal_connect(G_OBJECT(pf.entry1), "insert_text", G_CALLBACK(insert_text_handler_numeric), NULL);
		break;

	case GPID:
		ADD_DIALOG(_("GPID:"));
		g_signal_connect(G_OBJECT(pf.entry1), "insert_text", G_CALLBACK(insert_text_handler_numeric), NULL);
		break;
    }

    /*
	 * if "modify" button is pressed, fill dialogs above
	 * and run cycle for "modify".
	 */
	if (pf.modify_dialog_active == TRUE)
		modify_fill_dialog();
		
	gtk_widget_show_all (dialog);
		
	while ( gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT )
	{
		if (on_accept_dialog(dialog) == 0) break;
	}
		
	gtk_widget_destroy (dialog);
}

static gint on_accept_dialog (GtkWidget *dialog) {
	
	switch (pf.selected)
	{
		case PROCESSES:
		case IDLN:
		case FD:
		case PID:
		case GPID:
			return (check_new_input1 (dialog));
			break;
		
		case NETWORK:
			return (check_new_input2 (dialog));
			break;
		
		case DIR:
			return (check_new_input3 (dialog));
			break;
	}
}


/*
 * check_new_input's functions manage single input 
 * for process, fd, network, etc.
 */

/*
 * this check is for PROCESSES & IDLN & FD & PID & GPID
 */
static gint check_new_input1 (GtkWidget *dialog) {
	guint selected = pf.selected;
	const gchar *entry;
	gboolean state = FALSE;
	guint i;
	
	entry = gtk_entry_get_text (GTK_ENTRY(pf.entry1));
	if (strlen (entry) == 0) return -1;
			
	for (i=0; i < g_slist_length (pf.list[selected]); i++)
	{
		if (strcmp (entry, g_slist_nth_data (pf.list[selected], i)) == 0)
		{
			state = TRUE;
			break;
		}
	}
			
	if (state == FALSE)
	{
		if (pf.modify_dialog_active == TRUE)
		{
			GtkTreePath *path;
			gpointer element;
			guint root;
			
			/* show modified line in list-store */
			gtk_list_store_set (pf.liststore[selected], &(pf.iter_selected_row), 0, entry, -1);
			/* get number of a selected row in a list (PROCESSES, NETWORK, DIRECTORY, etc.) */
			path = gtk_tree_model_get_path(GTK_TREE_MODEL(pf.liststore[pf.selected]), &(pf.iter_selected_row));
			root = gtk_tree_path_get_indices(path)[0];
			gtk_tree_path_free(path);
			/* free element and remove it from array-list */
			element = g_slist_nth_data (pf.list[pf.selected], root);
			g_free (element);
			element = g_strdup (entry);
		}
		else
		{
			GtkTreeIter iter;
			
			gtk_list_store_append (pf.liststore[selected], &iter);
			gtk_list_store_set (pf.liststore[selected], &iter, 0, entry, -1);
			pf.list[selected] = g_slist_append (pf.list[selected], g_strdup (entry));
		}
		return 0;
	}
	else
	{
		WARNING_DIALOG (_("It's already included."), dialog);
		return -1;
	}
}


/*
 * this check is for NETWORK
 */
static gint check_new_input2 (GtkWidget *dialog) {
	guint selected = pf.selected;
	const gchar *entry1, *entry2;
	gchar *ipv, *protocol;
	network_data *nd;
	gboolean state = FALSE;
	GtkTreeIter iter;
	guint i, len;
	
	entry1 = gtk_entry_get_text (GTK_ENTRY(pf.entry1));
	entry2 = gtk_entry_get_text (GTK_ENTRY(pf.entry2));
	ipv = gtk_combo_box_get_active_text (GTK_COMBO_BOX(pf.combobox1));
	protocol = gtk_combo_box_get_active_text (GTK_COMBO_BOX(pf.combobox2));
	
	if ( (strlen (entry1) == 0) && (strlen (entry2) == 0) && (strlen(ipv) == 0) && (strlen(protocol) == 0) ) return -1;
	
	len = g_slist_length (pf.list[selected]);
	/* you can  not add more than 100 elements for network list */
	if (len > 99)
	{
		WARNING_DIALOG (_("Max. 100 elements reached."), dialog);
		return -1;
	}
	
	for (i=0; i < len; i++)
	{
		nd = g_slist_nth_data (pf.list[selected], i);
		if ( (g_ascii_strcasecmp (entry1, nd->host) == 0) && (g_ascii_strcasecmp (entry2, nd->port) == 0) \
			&& (g_ascii_strcasecmp (ipv, nd->ipv) == 0) && (g_ascii_strcasecmp (protocol, nd->protocol) == 0) )
		{
			state = TRUE;
			break;
		}
	}
			
	if (state == FALSE)
	{
		if (pf.modify_dialog_active == TRUE)
		{
			GtkTreePath *path;
			guint root;
			
			/* show modified line in list-store */
			gtk_list_store_set (pf.liststore[selected], &(pf.iter_selected_row), 0, entry1, 1, entry2, 2, ipv, 3, protocol, -1);
			/* get number of a selected row in a list (PROCESSES, NETWORK, DIRECTORY, etc.) */
			path = gtk_tree_model_get_path(GTK_TREE_MODEL(pf.liststore[pf.selected]), &(pf.iter_selected_row));
			root = gtk_tree_path_get_indices(path)[0];
			gtk_tree_path_free(path);
			/* free element and remove it from array-list */
			nd = g_slist_nth_data (pf.list[pf.selected], root);
			
			g_free (nd->host);
			g_free (nd->port);
			g_free (nd->ipv);
			g_free (nd->protocol);
			g_free (nd);
			
			nd = g_malloc(sizeof(network_data));
			nd->host = g_strdup (entry1);
			nd->port = g_strdup (entry2);
			nd->ipv = g_strdup (ipv);
			nd->protocol = g_strdup (protocol);
		}
		else
		{
			gtk_list_store_append (pf.liststore[selected], &iter);
			gtk_list_store_set (pf.liststore[selected], &iter, 0, entry1, 1, entry2, 2, ipv, 3, protocol, -1);
			nd = g_malloc(sizeof(network_data));
			nd->host = g_strdup (entry1);
			nd->port = g_strdup (entry2);
			nd->ipv = g_strdup (ipv);
			nd->protocol = g_strdup (protocol);
			pf.list[selected] = g_slist_append (pf.list[selected], nd);
		}
		return 0;
	}
	else
	{
		WARNING_DIALOG (_("It's already included."), dialog);
		return -1;
	}
}


/*
 * this check is for DIR
 */
static gint check_new_input3 (GtkWidget *dialog) {
	guint selected = pf.selected;
	const gchar *entry;
	dir_data *dd;
	gboolean state = FALSE;
	gboolean check_state;
	GtkTreeIter iter;
	guint i;
	
	entry = gtk_entry_get_text (GTK_ENTRY(pf.entry1));
	if (strlen (entry) == 0) return -1;
	check_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pf.check1));
			
	for (i=0; i < g_slist_length (pf.list[selected]); i++)
	{
		dd = g_slist_nth_data (pf.list[selected], i);
		if ( (strcmp (entry, dd->path) == 0) && (dd->descent == check_state) )
		{
			state = TRUE;
			break;
		}
	}
			
	if (state == FALSE)
	{
		if (pf.modify_dialog_active == TRUE)
		{
			GtkTreePath *path;
			guint root;
			
			/* get number of a selected row in a list (PROCESSES, NETWORK, DIRECTORY, etc.) */
			path = gtk_tree_model_get_path(GTK_TREE_MODEL(pf.liststore[pf.selected]), &(pf.iter_selected_row));
			root = gtk_tree_path_get_indices(path)[0];
			gtk_tree_path_free(path);
			/* free element and remove it from array-list */
			dd = g_slist_nth_data (pf.list[pf.selected], root);
			g_free (dd->path);
			g_free (dd);
			dd = g_malloc(sizeof(dir_data));
			dd->path = g_strdup (entry);
			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pf.check1)) == TRUE)
			{
				/* show modified line in list-store */
				gtk_list_store_set (pf.liststore[selected], &(pf.iter_selected_row), 0, entry, 1, _("Yes"), -1);
				dd->descent = TRUE;
			}
			else
			{
				/* show modified line in list-store */
				gtk_list_store_set (pf.liststore[selected], &(pf.iter_selected_row), 0, entry, 1, _("No"), -1);
				dd->descent = FALSE;
			}
		}
		else
		{
			gtk_list_store_append (pf.liststore[selected], &iter);
			dd = g_malloc(sizeof(dir_data));
			dd->path = g_strdup (entry);
			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pf.check1)) == TRUE)
			{
				gtk_list_store_set (pf.liststore[selected], &iter, 0, entry, 1, _("Yes"), -1);
				dd->descent = TRUE;
				pf.list[selected] = g_slist_append (pf.list[selected], dd);
			}
			else
			{
				gtk_list_store_set (pf.liststore[selected], &iter, 0, entry, 1, _("No"), -1);
				dd->descent = FALSE;
				pf.list[selected] = g_slist_append (pf.list[selected], dd);
			}
		}
		return 0;
	}
	else
	{
		WARNING_DIALOG (_("It's already included."), dialog);
		return -1;
	}
}


/*******************************************************************************
 *                                                                             *
 *                      MODIFY BUTTON SIGNAL & FUNCTIONS                       *
 *                                                                             *
 *******************************************************************************/


/*
 * modify row from a central list
 */
void on_modify_button_clicked (GtkButton * button, gpointer user_data)
{
	pf.modify_dialog_active = TRUE;
	on_add_button_clicked (NULL, NULL);
	pf.modify_dialog_active = FALSE;
}


static void modify_fill_dialog (void)
{
	gchar *entry[4];

	switch (pf.selected)
	{
		case PROCESSES:
		case IDLN:
		case FD:
		case PID:
		case GPID:
			gtk_tree_model_get (GTK_TREE_MODEL(pf.liststore[pf.selected]), &(pf.iter_selected_row), 0, &entry[0], -1);
			gtk_entry_set_text (GTK_ENTRY(pf.entry1), entry[0]);
			break;
		
		case NETWORK:
			gtk_tree_model_get (GTK_TREE_MODEL(pf.liststore[pf.selected]), &(pf.iter_selected_row), 0, &entry[0], 1, &entry[1], 2, &entry[2], 3, &entry[3], -1);
			gtk_entry_set_text (GTK_ENTRY(pf.entry1), entry[0]);
			gtk_entry_set_text (GTK_ENTRY(pf.entry2), entry[1]);
			
			if (strcmp (entry[2], _("4")) == 0)
				gtk_combo_box_set_active (GTK_COMBO_BOX(pf.combobox1), 1);
			else if (strcmp (entry[2], _("6")) == 0)
				gtk_combo_box_set_active (GTK_COMBO_BOX(pf.combobox1), 2);
			else
				gtk_combo_box_set_active (GTK_COMBO_BOX(pf.combobox1), 0);
			
			if (strcmp (entry[3], _("TCP")) == 0)
				gtk_combo_box_set_active (GTK_COMBO_BOX(pf.combobox2), 1);
			else if (strcmp (entry[3], _("UDP")) == 0)
				gtk_combo_box_set_active (GTK_COMBO_BOX(pf.combobox2), 2);
			else
				gtk_combo_box_set_active (GTK_COMBO_BOX(pf.combobox2), 0);
			break;
			
		case DIR:
			gtk_tree_model_get (GTK_TREE_MODEL(pf.liststore[pf.selected]), &(pf.iter_selected_row), 0, &entry[0], 1, &entry[1], -1);
			gtk_entry_set_text (GTK_ENTRY(pf.entry1), entry[0]);
			
			if (strcmp (entry[1], _("Yes")) == 0)
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pf.check1), TRUE);
			else
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pf.check1), FALSE);
			break;
	}
}


/*******************************************************************************
 *                                                                             *
 *                      REMOVE BUTTON SIGNAL & FUNCTIONS                       *
 *                                                                             *
 *******************************************************************************/


/*
 * remove row from a central list
 */
void on_remove_button_clicked (GtkButton * button, gpointer user_data)
{
	GtkTreePath *path;
	gpointer element;
	guint root;
	network_data *nd;
	dir_data *dd;
	
	/* get number of a selected row in a list (PROCESSES, NETWORK, DIRECTORY, etc.) */
	path = gtk_tree_model_get_path(GTK_TREE_MODEL(pf.liststore[pf.selected]), &(pf.iter_selected_row));
	root = gtk_tree_path_get_indices(path)[0];
	gtk_tree_path_free(path);
	/* free element and remove it from array-list */
	element = g_slist_nth_data (pf.list[pf.selected], root);
	pf.list[pf.selected] = g_slist_remove (pf.list[pf.selected], element);

	switch (pf.selected)
	{
		case PROCESSES:
		case IDLN:
		case FD:
		case PID:
		case GPID:
			g_free (element);
			break;
		
		case NETWORK:
			nd = element;
			g_free (nd->host);
			g_free (nd->port);
			g_free (nd->ipv);
			g_free (nd->protocol);
			g_free (nd);
			break;
		
		case DIR:
			dd = element;
			g_free (dd->path);
			g_free (dd);
			break;
	}
	
	gtk_list_store_remove(GTK_LIST_STORE(pf.liststore[pf.selected]), &(pf.iter_selected_row));
}


/*******************************************************************************
 *                                                                             *
 *                              MENU ITEMS SIGNALS                             *
 *                                                                             *
 *******************************************************************************/


void on_new_query_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	make_new_query ();
}



/*******************************************************************************
 *                                                                             *
 *                         LOAD, SAVE AND REMOVE QUERIES                       *
 *                                                                             *
 *******************************************************************************/


static void load_query(void)
{
	GSList *list_dir = NULL, *list_dir_depth = NULL, *iter_list = NULL;
	GSList *list_network_hostname = NULL, *list_network_ports = NULL, *list_network_ipv = NULL, *list_network_protocol = NULL;
	gchar *tmp_data[4];
	GtkTreeIter iter;
    guint i;
	dir_data *dd;
	network_data *nd;
    gchar *path = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(pf.gconf_query_basename, strlen(pf.gconf_query_basename)));

	/* main - Processes */
	pf.selected = PROCESSES;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_active[PROCESSES]), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_processes_list"), NULL));
	pf.list[PROCESSES] = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_processes"), GCONF_VALUE_STRING, NULL);
	for (i=0; i < g_slist_length (pf.list[PROCESSES]); i++)
	{
		gtk_list_store_append (pf.liststore[PROCESSES], &iter);
		gtk_list_store_set (pf.liststore[PROCESSES], &iter, 0, g_strdup ((gchar *)g_slist_nth_data (pf.list[PROCESSES],i)), -1);
	}
	
	/* main - IDLN */
	pf.selected = IDLN;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_active[IDLN]), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_idln_list"), NULL));
	pf.list[IDLN] = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_idln"), GCONF_VALUE_STRING, NULL);
	for (i=0; i < g_slist_length (pf.list[IDLN]); i++)
	{
		gtk_list_store_append (pf.liststore[IDLN], &iter);
		gtk_list_store_set (pf.liststore[IDLN], &iter, 0, g_strdup ((gchar *)g_slist_nth_data (pf.list[IDLN],i)), -1);
	}
	
	/* main - PID */
	pf.selected = PID;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_active[PID]), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_pid_list"), NULL));
	pf.list[PID] = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_pid"), GCONF_VALUE_STRING, NULL);
	for (i=0; i < g_slist_length (pf.list[PID]); i++)
	{
		gtk_list_store_append (pf.liststore[PID], &iter);
		gtk_list_store_set (pf.liststore[PID], &iter, 0, g_strdup ((gchar *)g_slist_nth_data (pf.list[PID],i)), -1);
	}
	
	/* main - GPID */
	pf.selected = GPID;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_active[GPID]), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_gpid_list"), NULL));
	pf.list[GPID] = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_gpid"), GCONF_VALUE_STRING, NULL);
	for (i=0; i < g_slist_length (pf.list[GPID]); i++)
	{
		gtk_list_store_append (pf.liststore[GPID], &iter);
		gtk_list_store_set (pf.liststore[GPID], &iter, 0, g_strdup ((gchar *)g_slist_nth_data (pf.list[GPID],i)), -1);
	}
	
	/* main - FD */
	pf.selected = FD;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_active[FD]), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_fd_list"), NULL));
	pf.list[FD] = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_fd"), GCONF_VALUE_STRING, NULL);
	for (i=0; i < g_slist_length (pf.list[FD]); i++)
	{
		gtk_list_store_append (pf.liststore[FD], &iter);
		gtk_list_store_set (pf.liststore[FD], &iter, 0, g_strdup ((gchar *)g_slist_nth_data (pf.list[FD],i)), -1);
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_exclude_fd), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "list_fd_exclude"), NULL));
	
	/* main - DIR */
	pf.selected = DIR;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_active[DIR]), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_dir_list"), NULL));
	list_dir = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_dir"), GCONF_VALUE_STRING, NULL);
	list_dir_depth = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_dir_depth"), GCONF_VALUE_STRING, NULL);
	for (i=0; i < g_slist_length (list_dir); i++)
	{
		tmp_data [0] = g_slist_nth_data (list_dir,i);
		tmp_data [1] = g_slist_nth_data (list_dir_depth,i);
		dd = g_malloc(sizeof(dir_data));
		dd->path = g_strdup (tmp_data[0]);
		
		if (strcmp (tmp_data[1], _("Yes")) == 0)
			dd->descent = TRUE;
		else
			dd->descent = FALSE;
		
		pf.list[DIR] = g_slist_append (pf.list[DIR], dd);
		gtk_list_store_append (pf.liststore[DIR], &iter);
		gtk_list_store_set (pf.liststore[DIR], &iter, 0, g_strdup (tmp_data[0]), 1, g_strdup (tmp_data[1]), -1);
	}
	
	FREE_LIST (list_dir);
	FREE_LIST (list_dir_depth);
		
	/* main - NETWORK */
	pf.selected = NETWORK;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.cb_active[NETWORK]), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_network_list"), NULL));
	list_network_hostname = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_hostnameip"), GCONF_VALUE_STRING, NULL);
	list_network_ports = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_portsservices"), GCONF_VALUE_STRING, NULL);
	list_network_ipv = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_ipv"), GCONF_VALUE_STRING, NULL);
	list_network_protocol = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_protocol"), GCONF_VALUE_STRING, NULL);
	for (i=0; i < g_slist_length (list_network_hostname); i++)
	{
		tmp_data [0] = g_slist_nth_data (list_network_hostname,i);
		tmp_data [1] = g_slist_nth_data (list_network_ports,i);
		tmp_data [2] = g_slist_nth_data (list_network_ipv,i);
		tmp_data [3] = g_slist_nth_data (list_network_protocol,i);
		
		nd = g_malloc(sizeof(network_data));
		nd->host = g_strdup (tmp_data[0]);
		nd->port = g_strdup (tmp_data[1]);
		nd->ipv = g_strdup (tmp_data[2]);
		nd->protocol = g_strdup (tmp_data[3]);
		
		pf.list[NETWORK] = g_slist_append (pf.list[NETWORK], nd);
		gtk_list_store_append (pf.liststore[NETWORK], &iter);
		gtk_list_store_set (pf.liststore[NETWORK], &iter, 0, g_strdup (tmp_data[0]), 1, g_strdup (tmp_data[1]), 2, g_strdup (tmp_data[2]), 3, g_strdup (tmp_data[3]), -1);
	}
	
	FREE_LIST (list_network_hostname);
	FREE_LIST (list_network_ports);
	FREE_LIST (list_network_ipv);
	FREE_LIST (list_network_protocol);
	
	/* START SITUATION */
	pf.selected = PROCESSES;

    /* general - id or login */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.id_radio), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "id"), NULL));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.login_radio), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "login"), NULL));

    /* general - offset or size */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.offset_radio), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "offset"), NULL));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.size_radio), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "size"), NULL));

    /* general - all AND */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.and_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path,	"and"), NULL));

    /* general - hide 'lsof' and 'glsof' from output */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.hideglsof_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "hideglsof"), NULL));

    /* processes - max number of links for a file */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.nlinks_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "numberlinks"), NULL));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(pf.nlinks_entry), (gdouble) gconf_client_get_int(client, gconf_concat_dir_and_key (path, "numberlinks_entry"), NULL));

    /* network - display ip-form addresses */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.ipform_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "ipadds"), NULL));

    /* network - port number */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.portnumbers_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path,	"ports"), NULL));

    /* network - nfs */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.nfs_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path,	"nfs"), NULL));

    /* network - unix sockets */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.unixsockets_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "unixsockets"), NULL));

    /* kernel - timeout seconds */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.timeout_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "timeoutsecs"), NULL));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(pf.timeout_entry), (gdouble) gconf_client_get_int(client, gconf_concat_dir_and_key (path, "timeoutsecs_entry"), NULL));

    /* kernel - avoid */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pf.avoid_check), gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "avoid"), NULL));

    g_free(path);
}


static void save_query(void)
{
    GSList *list_dir = NULL, *list_dir_depth = NULL, *iter_list = NULL;
	GSList *list_network_hostname = NULL, *list_network_ports = NULL, *list_network_ipv = NULL, *list_network_protocol = NULL;
	dir_data *dd;
	network_data *nd;
    guint i;
	gchar *path = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(pf.gconf_query_basename, strlen(pf.gconf_query_basename)));

	/* main - processes */
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "active_processes_list"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[PROCESSES])), NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_processes"), GCONF_VALUE_STRING, pf.list[PROCESSES], NULL);
	
	/* main - idln */
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "active_idln_list"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[IDLN])), NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_idln"), GCONF_VALUE_STRING, pf.list[IDLN], NULL);
	
	/* main - pid */
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "active_pid_list"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[PID])), NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_pid"), GCONF_VALUE_STRING, pf.list[PID], NULL);
	
	/* main - gpid */
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "active_gpid_list"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[GPID])), NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_gpid"), GCONF_VALUE_STRING, pf.list[GPID], NULL);
	
	/* main - fd */
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "active_fd_list"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[FD])), NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_fd"), GCONF_VALUE_STRING, pf.list[FD], NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "list_fd_exclude"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_exclude_fd)), NULL);
	
	/* main - dir */
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "active_dir_list"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[DIR])), NULL);
	
	for (i=0; i < g_slist_length (pf.list[DIR]); i++)
	{
		dd = g_slist_nth_data (pf.list[DIR],i);
		list_dir = g_slist_append (list_dir, g_strdup(dd->path));
		
		if (dd->descent == TRUE)
			list_dir_depth = g_slist_append (list_dir_depth, _("Yes"));
		else
			list_dir_depth = g_slist_append (list_dir_depth, _("No"));
	}
	
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_dir"), GCONF_VALUE_STRING, list_dir, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_dir_depth"), GCONF_VALUE_STRING, list_dir_depth, NULL);
	
	/* main - network */
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "active_network_list"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.cb_active[NETWORK])), NULL);
	
	for (i=0; i < g_slist_length (pf.list[NETWORK]); i++)
	{
		nd = g_slist_nth_data (pf.list[NETWORK],i);
		list_network_hostname = g_slist_append (list_network_hostname, g_strdup(nd->host));
		list_network_ports = g_slist_append (list_network_ports, g_strdup(nd->port));
		list_network_ipv = g_slist_append (list_network_ipv, g_strdup(nd->ipv));
		list_network_protocol = g_slist_append (list_network_protocol, g_strdup(nd->protocol));
	}

	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_network_hostnameip"), GCONF_VALUE_STRING, list_network_hostname, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_network_portsservices"), GCONF_VALUE_STRING, list_network_ports, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_network_ipv"), GCONF_VALUE_STRING, list_network_ipv, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path, "list_network_protocol"), GCONF_VALUE_STRING, list_network_protocol, NULL);

    /* general - id or login */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "id"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.id_radio)), NULL);
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "login"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.login_radio)), NULL);

    /* general - offset or size */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "offset"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.offset_radio)), NULL);
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "size"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.size_radio)), NULL);

    /* general - all AND */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "and"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.and_check)), NULL);

    /* general - hide 'lsof' and 'glsof' from output */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "hideglsof"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.hideglsof_check)), NULL);

    /* processes - max number of links for a file */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "numberlinks"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.nlinks_check)), NULL);
    gconf_client_set_int(client, gconf_concat_dir_and_key(path, "numberlinks_entry"), gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (pf.nlinks_entry)), NULL);

    /* network - display ip-form addresses */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "ipadds"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.ipform_check)), NULL);

    /* network - port number */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "ports"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.portnumbers_check)), NULL);

    /* network - nfs */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "nfs"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.nfs_check)), NULL);

    /* network - unix sockets */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "unixsockets"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.unixsockets_check)), NULL);

    /* kernel - timeout seconds */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "timeoutsecs"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.timeout_check)), NULL);
    gconf_client_set_int(client, gconf_concat_dir_and_key(path, "timeoutsecs_entry"), gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (pf.timeout_entry)), NULL);

    /* kernel - avoid */
    gconf_client_set_bool(client, gconf_concat_dir_and_key(path, "avoid"), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (pf.avoid_check)), NULL);
    
    gconf_client_suggest_sync (client, NULL);

    g_free(path);
}


static void copy_query(const gchar *old_name, const gchar *new_name)
{
	gboolean state, state2;
	GSList *list, *list2, *list3, *list4;
	gint value;
    gchar *path = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(old_name, strlen(old_name)));
	gchar *path2 = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(new_name, strlen(new_name)));
	
	/* main - Processes */
	state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_processes_list"), NULL);
	list  = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_processes"), GCONF_VALUE_STRING, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "active_processes_list"), state, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_processes"), GCONF_VALUE_STRING, list, NULL);
	FREE_LIST (list);
	
	/* main - IDLN */
	state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_idln_list"), NULL);
	list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_idln"), GCONF_VALUE_STRING, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "active_idln_list"), state, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_idln"), GCONF_VALUE_STRING, list, NULL);
	FREE_LIST (list);
	
	/* main - PID */
	state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_pid_list"), NULL);
	list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_pid"), GCONF_VALUE_STRING, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "active_pid_list"), state, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_pid"), GCONF_VALUE_STRING, list, NULL);
	FREE_LIST (list);
	
	/* main - GPID */
	state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_gpid_list"), NULL);
	list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_gpid"), GCONF_VALUE_STRING, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "active_gpid_list"), state, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_gpid"), GCONF_VALUE_STRING, list, NULL);
	FREE_LIST (list);
	
	/* main - FD */
	state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_fd_list"), NULL);
	list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_fd"), GCONF_VALUE_STRING, NULL);
	state2 = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "list_fd_exclude"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "active_fd_list"), state, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_fd"), GCONF_VALUE_STRING, list, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "list_fd_exclude"), state2, NULL);
	FREE_LIST (list);
	
	/* main - DIR */
	state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_dir_list"), NULL);
	list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_dir"), GCONF_VALUE_STRING, NULL);
	list2 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_dir_depth"), GCONF_VALUE_STRING, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "active_dir_list"), state, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_dir"), GCONF_VALUE_STRING, list, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_dir_depth"), GCONF_VALUE_STRING, list2, NULL);
	FREE_LIST (list);
	FREE_LIST (list2);
		
	/* main - NETWORK */
	state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_network_list"), NULL);
	list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_hostnameip"), GCONF_VALUE_STRING, NULL);
	list2 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_portsservices"), GCONF_VALUE_STRING, NULL);
	list3 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_ipv"), GCONF_VALUE_STRING, NULL);
	list4 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_protocol"), GCONF_VALUE_STRING, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "active_network_list"), state, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_network_hostnameip"), GCONF_VALUE_STRING, list, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_network_portsservices"), GCONF_VALUE_STRING, list2, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_network_ipv"), GCONF_VALUE_STRING, list3, NULL);
	gconf_client_set_list(client, gconf_concat_dir_and_key(path2, "list_network_protocol"), GCONF_VALUE_STRING, list4, NULL);
	FREE_LIST (list);
	FREE_LIST (list2);
	FREE_LIST (list3);
	FREE_LIST (list4);
	
	/* general - id or login */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "id"), NULL);
    state2 = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "login"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "id"), state, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "login"), state2, NULL);
	
    /* general - offset or size */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "offset"), NULL);
    state2 = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "size"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "offset"), state, NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "size"), state2, NULL);

    /* general - all AND */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "and"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "and"), state, NULL);

    /* general - hide 'lsof' and 'glsof' from output */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "hideglsof"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "hideglsof"), state, NULL);

    /* processes - max number of links for a file */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "numberlinks"), NULL);
    value = gconf_client_get_int(client, gconf_concat_dir_and_key (path, "numberlinks_entry"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "numberlinks"), state, NULL);
	gconf_client_set_int(client, gconf_concat_dir_and_key(path2, "numberlinks_entry"), value, NULL);
	
	/* processes - autorefresh */
	state = gconf_client_get_bool (client, gconf_concat_dir_and_key(path, "autorefresh"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "autorefresh"), state, NULL);

    /* network - display ip-form addresses */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "ipadds"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "ipadds"), state, NULL);

    /* network - port number */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "ports"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "ports"), state, NULL);

    /* network - nfs */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "nfs"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "nfs"), state, NULL);

    /* network - unix sockets */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "unixsockets"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "unixsockets"), state, NULL);

    /* kernel - timeout seconds */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "timeoutsecs"), NULL);
    value = gconf_client_get_int(client, gconf_concat_dir_and_key (path, "timeoutsecs_entry"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "timeoutsecs"), state, NULL);
	gconf_client_set_int(client, gconf_concat_dir_and_key(path2, "timeoutsecs_entry"), value, NULL);

    /* kernel - avoid */
    state = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "avoid"), NULL);
	gconf_client_set_bool(client, gconf_concat_dir_and_key(path2, "avoid"), state, NULL);
	
	gconf_client_suggest_sync (client, NULL);
	
	g_free(path);
	g_free(path2);
}


static void remove_query(void)
{
    gchar *path = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(pf.gconf_query_basename, strlen(pf.gconf_query_basename)));
    
    /* main - processes */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "active_processes_list"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_processes"), NULL);
	
	/* main - idln */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "active_idln_list"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_idln"), NULL);
	
	/* main - pid */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "active_pid_list"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_pid"), NULL);
	
	/* main - gpid */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "active_gpid_list"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_gpid"), NULL);
	
	/* main - fd */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "active_fd_list"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_fd"), NULL);
	gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_fd_exclude"), NULL);
	
	/* main - dir */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "active_dir_list"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_dir"), NULL);
	gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_dir_depth"), NULL);
	
	/* main - network */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "active_network_list"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_network_hostnameip"), NULL);
	gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_network_portsservices"), NULL);
	gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_network_ipv"), NULL);
	gconf_client_unset(client, gconf_concat_dir_and_key(path, "list_network_protocol"), NULL);

    /* general - id or login */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "id"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "login"), NULL);

    /* general - offset or size */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "offset"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "size"), NULL);

    /* general - all AND */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "and"), NULL);

    /* general - hide 'lsof' and 'glsof' from output */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "hideglsof"), NULL);

    /* files - processes name exec */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "procexe"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "procexe_entry"), NULL);

    /* files - processes ID */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "procid"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "procid_entry"), NULL);

    /* files - processes PGID */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "procpgid"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "procpgid_entry"), NULL);

    /* processes - display directory */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "disdir"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "disdir_entry"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "depth"), NULL);

    /* processes - files descriptors */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "fdlistdesc"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "fdlistdesc_entry"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "incexc"), NULL);

    /* processes - list of files for id or username */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "listidname"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "listidname_entry"), NULL);

    /* processes - max number of links for a file */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "numberlinks"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "numberlinks_entry"), NULL);
	
	/* processes - autorefresh */
	gconf_client_unset(client, gconf_concat_dir_and_key(path, "autorefresh"), NULL);

    /* network - display ip-form addresses */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "ipadds"), NULL);

    /* network - port number */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "ports"), NULL);

    /* network - nfs */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "nfs"), NULL);

    /* network - unix sockets */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "unixsockets"), NULL);

    /* kernel - timeout seconds */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "timeoutsecs"), NULL);
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "timeoutsecs_entry"), NULL);

    /* kernel - avoid */
    gconf_client_unset(client, gconf_concat_dir_and_key(path, "avoid"), NULL);
    
    gconf_client_suggest_sync (client, NULL);

    g_free(path);
}
