/*  refresh - refresh single and double layer table, and check lsof's options.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <stdio.h>

#include "query_editor.h"
#include "interface.h"
#include "popupmenu.h"
#include "refresh.h"
#include "double_list_main_object.h"

/* interface */
extern main_window interface;

/* main */
extern GConfClient *client;
extern gchar *lsof_path;

/* query_editor */
extern query_editor pf;

/* extern */
gchar tcp[12][BUFSIZ];


/************************************/
/*      CONTROL FOR SETTINGS        */
/************************************/

static gchar *control_settings(void)
{
	guint i;
	GSList *list = NULL, *list2 = NULL, *list3 = NULL, *list4 = NULL;
	gchar buf [G_ASCII_DTOSTR_BUF_SIZE];	/* converted number to string */
	gchar *commands_lsof;
	gchar **split_array;
	gchar *path;
	
	path = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(pf.gconf_query_basename, strlen(pf.gconf_query_basename)));	/* path for gconf query */

	commands_lsof = " ";

/***************/
/*   General   */
/***************/

/* -l */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "id"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -l ", NULL);

/* -o -s */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "offset"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -o ", NULL);
	else commands_lsof = g_strconcat(commands_lsof, " -s ", NULL);

/* -a */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "and"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -a ", NULL);


/*************/
/*   Files   */
/*************/

/* -c */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_processes_list"), NULL))
	{
		list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_processes"), GCONF_VALUE_STRING, NULL);
		for (i=0; i < g_slist_length (list); i++)
		{
			commands_lsof = g_strconcat(commands_lsof, " -c ", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
		}
	}
	FREE_LIST (list);


/* -p */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_pid_list"), NULL))
	{
		list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_pid"), GCONF_VALUE_STRING, NULL);
		commands_lsof = g_strconcat(commands_lsof, " -p ", g_strdup ((gchar *)g_slist_nth_data (list,0)), NULL);
		for (i=1; i < g_slist_length (list); i++)
		{
			commands_lsof = g_strconcat(commands_lsof, ",", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
		}
	}
	FREE_LIST (list);


/* -g */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_gpid_list"), NULL))
	{
		list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_gpid"), GCONF_VALUE_STRING, NULL);
		commands_lsof = g_strconcat(commands_lsof, " -g ", g_strdup ((gchar *)g_slist_nth_data (list,0)), NULL);
		for (i=1; i < g_slist_length (list); i++)
		{
			commands_lsof = g_strconcat(commands_lsof, ",", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
		}
	}
	FREE_LIST (list);


/*****************/
/*   Processes   */
/*****************/

/* +d +D */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_dir_list"), NULL))
	{
		list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_dir"), GCONF_VALUE_STRING, NULL);
		list2 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_dir_depth"), GCONF_VALUE_STRING, NULL);
		for (i=0; i < g_slist_length (list); i++)
		{
			if (strcmp (g_slist_nth_data (list2,i), _("Yes")) == 0)
				commands_lsof = g_strconcat(commands_lsof, " +D ", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
			else
				commands_lsof = g_strconcat(commands_lsof, " +d ", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
		}
	}
	FREE_LIST (list);
	FREE_LIST (list2);

/* -d */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_fd_list"), NULL))
	{
		list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_fd"), GCONF_VALUE_STRING, NULL);
		commands_lsof = g_strconcat(commands_lsof, " -d ", NULL);
		
		if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "list_fd_exclude"), NULL))
		{
			commands_lsof = g_strconcat(commands_lsof, "^", g_strdup ((gchar *)g_slist_nth_data (list,0)), NULL);
			for (i=1; i < g_slist_length (list); i++)
			{
				commands_lsof = g_strconcat(commands_lsof, ",^", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
			}
		}
		else
		{
			commands_lsof = g_strconcat(commands_lsof, g_strdup ((gchar *)g_slist_nth_data (list,0)), NULL);
			for (i=1; i < g_slist_length (list); i++)
			{
				commands_lsof = g_strconcat(commands_lsof, ",", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
			}
		}
	}
	FREE_LIST (list);

/* -u */
	if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_idln_list"), NULL))
	{
		list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_idln"), GCONF_VALUE_STRING, NULL);
		commands_lsof = g_strconcat(commands_lsof, " -u ", g_strdup ((gchar *)g_slist_nth_data (list,0)), NULL);
		for (i=1; i < g_slist_length (list); i++)
		{
			commands_lsof = g_strconcat(commands_lsof, ",", g_strdup ((gchar *)g_slist_nth_data (list,i)), NULL);
		}
	}
	FREE_LIST (list);

/* +L */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "numberlinks"), NULL)) 
	{
		gdouble val = (gdouble) gconf_client_get_int(client, gconf_concat_dir_and_key (path, "numberlinks_entry"), NULL);
		g_ascii_dtostr (&buf[0], G_ASCII_DTOSTR_BUF_SIZE, val);
		commands_lsof = g_strconcat(commands_lsof, " +L ", buf, NULL);
	}


/***************/
/*   Network   */
/***************/

/* -i */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "active_network_list"), NULL))
	{
		guint len,l1,l2,l3,l4;
		
		list = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_hostnameip"), GCONF_VALUE_STRING, NULL);
		list2 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_portsservices"), GCONF_VALUE_STRING, NULL);
		list3 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_ipv"), GCONF_VALUE_STRING, NULL);
		list4 = gconf_client_get_list (client, gconf_concat_dir_and_key (path, "list_network_protocol"), GCONF_VALUE_STRING, NULL);
		
		/* ugly most great value:) */
		l1 = g_slist_length(list);
		l2 = g_slist_length(list2);
		l3 = g_slist_length(list3);
		l4 = g_slist_length(list4);
		len = l1;
		if (len < l2) len = l2;
		if (len < l3) len = l3;
		if (len < l4) len = l4;
		
		if ( (l1==0) && (l2==0) && (l3==0) && (l4==0) )
			commands_lsof = g_strconcat(commands_lsof, " -i ", NULL);
		else
		{
			gchar *part = NULL;
			
			for (i=0; i < len; i++)
			{
				commands_lsof = g_strconcat(commands_lsof, " -i ", NULL);

				part = (gchar *)g_slist_nth_data (list3,i);
				if (strcmp (part, "") != 0)
					commands_lsof = g_strconcat(commands_lsof, part, NULL);

				part = (gchar *)g_slist_nth_data (list4,i);
				if (strcmp (part, "") != 0)
					commands_lsof = g_strconcat(commands_lsof, part, NULL);

				part = (gchar *)g_slist_nth_data (list,i);
				if (strcmp (part, "") != 0)
					commands_lsof = g_strconcat(commands_lsof, "@", part, NULL);

				part = (gchar *)g_slist_nth_data (list2,i);
				if (strcmp (part, "") != 0)
					commands_lsof = g_strconcat(commands_lsof, ":", part, NULL);
			}
		}
	}
	FREE_LIST (list);
	FREE_LIST (list2);
	FREE_LIST (list3);
	FREE_LIST (list4);

/* -n */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "ipadds"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -n ", NULL);

/* -N */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "nfs"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -N ", NULL);

/* -P */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "ports"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -P ", NULL);

/* -U */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "unixsockets"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -U ", NULL);


/**************/
/*   Kernel   */
/**************/

/* -S */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "timeoutsecs"), NULL))
	{
		gdouble val = (gdouble) gconf_client_get_int(client, gconf_concat_dir_and_key (path, "timeoutsecs_entry"), NULL);
		g_ascii_dtostr (&buf[0], G_ASCII_DTOSTR_BUF_SIZE, val);
		commands_lsof = g_strconcat(commands_lsof, " -S ", buf, NULL);
	}

/* -b */
    if (gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "avoid"), NULL)) commands_lsof = g_strconcat(commands_lsof, " -b ", NULL);
    
    g_free (path);

/************************/
/* remove double spaces */
/************************/
	
	g_strstrip(commands_lsof);
	split_array = g_strsplit(commands_lsof, "  ", -1);
	g_free(commands_lsof);
	commands_lsof = g_strjoinv(" ", split_array);
	g_strfreev(split_array);
	commands_lsof = g_strconcat(lsof_path, commands_lsof, NULL);
    
	return commands_lsof;
}


void update_clist(guint index_query)
{
    static gchar readbuf[BUFSIZ];
    static gchar readbuf2[12][BUFSIZ];
    static gchar fd[10];
    
    FILE *fp, *efp;
    gchar **filter_argv;
    GtkTextIter iter;
    gchar *commands_lsof = NULL;
    gint output = 0, error = 2;
    gint mod = 1000;
    gboolean print = FALSE;
    gchar *path;
    gboolean hide;
    
    path = gconf_concat_dir_and_key (GCONF_PATH_QUERY, gconf_escape_key(pf.gconf_query_basename, strlen(pf.gconf_query_basename)));	/* path for gconf query */
    hide = gconf_client_get_bool(client, gconf_concat_dir_and_key (path, "hideglsof"), NULL);
    g_free (path);

    /* start control */
    commands_lsof = control_settings();

    filter_argv = g_strsplit(commands_lsof, " ", -1);
    
    memset(fd, 0, sizeof(fd));
    memset(readbuf2, 0, sizeof(readbuf2));
    memset(tcp, 0, sizeof(tcp));

    /* fork 'lsof' */
    g_spawn_async_with_pipes(NULL, filter_argv, NULL, 0, NULL, NULL, NULL, NULL, &output, &error, NULL);
    
    fp = fdopen(output, "r");
    
    efp = fdopen(error, "r");
    
    g_strfreev(filter_argv);

    while (fgets(readbuf, BUFSIZ, fp)) {

	if (gtk_events_pending()) gtk_main_iteration();	/* refresh window */

	switch (readbuf[0]) {

	case 'c':
	    {
			mod = 0;
			break;
	    }			/* COMMAND */
	case 'p':
	    {
			mod = 1;
			break;
	    }			/* PID */
	case 'g':
	    {
			mod = 2;
			break;
	    }			/* PGID */
	case 'R':
	    {
			mod = 3;
			break;
	    }			/* PPID */
	case 'u':
	    {
			mod = 4;
			break;
	    }			/* USER */
	case 'L':
	    {
			mod = 4;
			break;
	    }
	case 'f':
	    {
			mod = 5;
			strncat(&fd[0], &readbuf[1], strlen(readbuf) - 2);
			break;
	    }			/* FD */
	case 'a':
	    {
			mod = 5;
			strncat(&fd[0], &readbuf[1], strlen(readbuf) - 2);
			break;
	    }
	case 'l':
	    {
			mod = 5;
			strncat(&fd[0], &readbuf[1], strlen(readbuf) - 2);
			break;
	    }
	case 't':
	    {
			mod = 6;
			break;
	    }			/* TYPE */
	case 'd':
	    {
			mod = 7;
			break;
	    }			/* DEVICE */
	case 'D':
	    {
			mod = 7;
			break;
	    }
	case 'o':
	    {
			mod = 8;
			break;
	    }			/* SIZE/OFF */
	case 's':
	    {
			mod = 8;
			break;
	    }
	case 'k':
	    {
			mod = 9;
			break;
	    }			/* NLINK */
	case 'i':
	    {
			mod = 10;
			break;
	    }			/* NODE */
	case 'P':
	    {
			mod = 10;
			break;
	    }
	case 'S':
	    {
			mod = 11;
			print = TRUE;
			break;
	    }			/* NAME */
	case 'n':
	    {
			mod = 11;
			print = TRUE;
			break;
	    }
	case 'T':
	    {
			strcat(tcp[11], "(");
			strncat(tcp[11], &readbuf[1], strlen(readbuf) - 2);
			strcat(tcp[11], ")");
			break;
	    }

	default:
	    mod = 1000;
	}

	if (mod < 1000 && mod != 5) {
	    memset(readbuf2[mod], 0, sizeof(readbuf2[mod]));
	    strncpy(&readbuf2[mod][0], &readbuf[1], strlen(readbuf) - 2);
	    mod = 1000;
	}

	if (print == TRUE) {
	    guint i;

	    strncpy(&readbuf2[5][0], &fd[0], strlen(fd));
	    memset(fd, 0, sizeof(fd));

	    if (strlen(tcp[0]) > 0) {
			gboolean add_line_table = TRUE;

			if (hide)
				if ((!strcmp(tcp[0], "glsof")) || (!strcmp(tcp[0], "lsof"))) add_line_table = FALSE;

			if (add_line_table == TRUE) double_list_main_add_row(DOUBLE_LIST_MAIN(interface.dlm), index_query);
	    }

	    print = FALSE;
	    for (i = 0; i < 12; i++) strcpy(&tcp[i][0], &readbuf2[i][0]);
	    for (i = 5; i < 12; i++) memset(readbuf2[i], 0, sizeof(readbuf2[i]));
	}
	/* _if (print) */
	memset(readbuf, 0, sizeof(readbuf));

    }				/* _while */

    if ((strlen(readbuf) == 0) && (strlen(tcp[0]) != 0))
	if ((strcmp(tcp[0], "glsof")) && (strcmp(tcp[0], "lsof")))
	    double_list_main_add_row(DOUBLE_LIST_MAIN(interface.dlm), index_query);
	    
    fclose(fp);
    memset(readbuf, 0, sizeof(readbuf));
    
    /* print on debug-output textview */
    gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, -1);
    
    gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, _("Query: "), strlen(_("Query: ")), "heading", "red_foreground", NULL);
    gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, pf.gconf_query_basename, strlen(pf.gconf_query_basename), "heading", "red_foreground", NULL);
    gtk_text_buffer_insert (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, "\n", 1);
    
    gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, commands_lsof, strlen(commands_lsof), "bold", NULL);
    gtk_text_buffer_insert (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, "\n", 1);
    
    while (fgets(readbuf, BUFSIZ, efp))
    {
    	gtk_text_buffer_insert (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, readbuf, strlen(readbuf));
    }
    
    gtk_text_buffer_insert (GTK_TEXT_BUFFER(interface.debug_output_text_buffer), &iter, "\n", 1);
	
    fclose(efp);
    g_free(commands_lsof);
}
