/*  search - search datas in the list.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */


#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "interface.h"
#include "refresh.h"
#include "double_list_main_object.h"
#include "search.h"

save_search_position ssp;

/* interface */
extern main_window interface;

/* extern */
gchar *name = NULL;
gchar *name_copy = NULL;

/* static */
static gboolean found_string = FALSE;
static gboolean end_loop = FALSE;
		
		
/*******************************************************************************
 *                                                                             *
 *                           INPUT DIALOG FUNCTIONS                            *
 *                                                                             *
 *******************************************************************************/

void on_search_entry_changed(GtkEditable * editable, gpointer data)
{
    gboolean state = FALSE;

    name = gtk_editable_get_chars(editable, 0, -1);

    if (*name != '\0')
		state = TRUE;

    gtk_widget_set_sensitive(interface.combox, state);
    gtk_widget_set_sensitive(interface.search_button, state);
}

/* dialog restart */
static void restart_confirm_response(GtkWidget * dialog, int response_id, GtkWindow * transient_parent)
{
    if (response_id == GTK_RESPONSE_YES)
	{
		found_string = FALSE;
		SAVE_SEARCH_STATUS (0, 0, 0, 0);
		on_search_button_clicked (NULL, NULL);
	}
}

static gboolean dialog_restart(void)
{
    GtkWidget *dialog;

    /* restart: YES or NO */
    dialog = gtk_message_dialog_new(GTK_WINDOW(interface.window),
				    GTK_DIALOG_DESTROY_WITH_PARENT,
				    GTK_MESSAGE_QUESTION,
				    GTK_BUTTONS_NONE,
				    _("%s was not found. Wrap search around the list?"),
				    name);

    gtk_dialog_add_buttons(GTK_DIALOG(dialog),
			   GTK_STOCK_NO,
			   GTK_RESPONSE_NO,
			   GTK_STOCK_YES, GTK_RESPONSE_YES, NULL);

    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);

    gtk_window_set_title(GTK_WINDOW(dialog), _("Searching..."));
    gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);

    g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(restart_confirm_response), NULL);

    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);

    return TRUE;
}


/*******************************************************************************
 *                                                                             *
 *                              SEARCH FUNCTIONS                               *
 *                                                                             *
 *******************************************************************************/

static void search_string (gint p, guint number_queries, gint index_column)
{
	gint query;
	guint i, j, k, l, m, number_processes, number_lines_for_process;
	gchar *column[11];
	gchar *process;
	GtkListStore *liststore;
	GtkTreeSelection *selection;
	GtkTreeIter iter;
	GtkTreeView *treeview;
	GtkTreePath *path;
	static gboolean process_already_selected = FALSE;
	
	/* load values from saved search position */
	i = ssp.save_query_position;
	j = ssp.save_process_position;
	k = ssp.save_process_line_position;
	l = ssp.save_process_line_column_position;
	
	query = p;
	
	for (; i < number_queries; i++)
	{
		number_processes = double_list_main_get_number_processes_for_query (interface.dlm, i);
		
		for (; j < number_processes; j++)
		{
			process = double_list_main_get_name_process_for_query (interface.dlm, i, j);
			
			if (g_strrstr(process, name))
			{
				if (process_already_selected == FALSE)
				{
					/* matched */
					double_list_main_select_process_row_in_query (interface.dlm, i, j);
				
					SAVE_SEARCH_STATUS (i, j, k, l);
					
					process_already_selected = TRUE;
					
					/* search is not finished */
					found_string = TRUE;
				
					return;
				}
			}
			else
			{
				process_already_selected = FALSE;
			}
			
			if (index_column != -1)	/* not process searching */
			{
				number_lines_for_process = double_list_main_get_number_lines_process_for_query (interface.dlm, i, j);
			
				liststore = double_list_main_get_process_liststore_for_query (interface.dlm, i, j);
				treeview = double_list_main_get_process_treeview_for_query(interface.dlm, i, j);
				selection = gtk_tree_view_get_selection (treeview);

				for (; k < number_lines_for_process; k++)
				{
					gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(liststore), &iter, NULL, k);
				
					if (index_column == -2)	/* searching in everywhere... */
					{
						gtk_tree_model_get(GTK_TREE_MODEL(liststore), &iter, \
							0, &column[0], 1, &column[1], 2, &column[2], 3, &column[3], 4, &column[4], 5, &column[5], 6, &column[6], 7, &column[7], 8, &column[8], 9, &column[9], 10, &column[10], -1);
				
						for (; l < 11; l++)
						{
							if (g_strrstr(column[l], name))
							{
								/* matched */
								double_list_main_select_process_row_in_query (interface.dlm, i, j);
								for (m = 0; m < 11; m++) g_free(column[m]);
							
								/* select and scroll to row */
								gtk_tree_selection_select_iter (selection, &iter);
								path = gtk_tree_model_get_path (GTK_TREE_MODEL(liststore), &iter);
								gtk_tree_view_scroll_to_cell (treeview, path, NULL, NULL, 0.0, 0.0);
								gtk_tree_path_free(path);
							
								SAVE_SEARCH_STATUS (i, j, k+1, 0);
						
								/* search is not finished */
								found_string = TRUE;
							
								return;
							}
						}
						l=0;
					}
					else	/* searching in specified column */
					{
						gtk_tree_model_get(GTK_TREE_MODEL(liststore), &iter, index_column, &column[index_column], -1);
					
						if (g_strrstr(column[index_column], name))
						{
							/* matched */
							double_list_main_select_process_row_in_query (interface.dlm, i, j);
							g_free(column[index_column]);
							
							/* select and scroll to row */
							gtk_tree_selection_select_iter (selection, &iter);
							path = gtk_tree_model_get_path (GTK_TREE_MODEL(liststore), &iter);
							gtk_tree_view_scroll_to_cell (treeview, path, NULL, NULL, 0.0, 0.0);
							gtk_tree_path_free(path);
						
							SAVE_SEARCH_STATUS (i, j, k+1, 0);
						
							/* search is not finished */
							found_string = TRUE;
						
							return;
						}
					}
				
					if (index_column == -2)
						for (m = 0; m < 11; m++) g_free(column[m]);
					else
						g_free(column[index_column]);
				}
				k=0;
			}
		}
		j=0;
	}
	
	/* search finished */
	end_loop = TRUE;
	
	return;
}

/* main search routine */
void on_search_button_clicked(GtkButton * button, gpointer user_data)
{
	guint i, number_queries;
	gint option;
	gint query = -1;
	static gint option_copy = -1;
	
	number_queries = double_list_main_get_number_queries (interface.dlm);
	
	/* start from the first query not empty */
	for (query=0; query < number_queries; query++)
	{
		if (double_list_main_get_number_processes_for_query (interface.dlm, query) !=0)
		{
			break;
		}
	}
	
	/* get option from combobox */
	option = gtk_combo_box_get_active(GTK_COMBO_BOX(interface.combox));

	if (name_copy == NULL)	/* first search */
	{
		found_string = FALSE;
		SAVE_SEARCH_STATUS (0, 0, 0, 0);
		name_copy = g_strdup(name);
		option_copy = option;
	}
	else if (strcmp(name, name_copy) != 0)	/* have you changed string? */
	{
		found_string = FALSE;
		SAVE_SEARCH_STATUS (0, 0, 0, 0);
		g_free(name_copy);
		name_copy = g_strdup(name);
		option_copy = option;
	}
	else if (option != option_copy)	/* have you changed combobox-option? */
	{
		found_string = FALSE;
		SAVE_SEARCH_STATUS (0, 0, 0, 0);
		option_copy = option;
	}
		
	search_string(query, number_queries, option - 2);
		
	/* if one row is selected launch 'restart' dialog */
	if (end_loop == TRUE)
	{
		end_loop = FALSE;
			
		if (found_string == TRUE)
		{
			dialog_restart();
		}
		else
		{
			ERROR_DIALOG(_("%s could not be found."), name);
		}
	}
}
